<?php


namespace Photobooth;


class FileUploader {

    /**
     * @var FileManagement
     */
    private $file_management;

    public function getFileManagement() {

        return $this->file_management;
    }

    public function setFileManagement( $file_management ) {

        $this->file_management = $file_management;
    }


    public function handle_upload_file() {

        $settings_id = intval( sanitize_text_field( $_GET[ 'settings-id' ] ?? 0 ) );

        $valid_upload = get_query_var( 'pg-upload', false );

        if ( $valid_upload === false ) {
            return;
        }

        if ( $settings_id <= 0 && strpos( $valid_upload, '-' ) !== false ) {

            list( $valid_upload, $settings_id ) = explode( '-', $valid_upload );

            $settings_id = intval( $settings_id );
        }

        if ( $settings_id <= 0 ) {
            Helper::log( sprintf( '[ Webhook Uploader ] HTTP 403 Forbidden - Settings ID not defined' ), 'error' );
            header( "HTTP/1.0 403 Forbidden" );
            die();
        }

        $password = get_field( 'webhook_upload_password', $settings_id );

        if ( $valid_upload !== $password ) {
            Helper::log( sprintf( '[ Webhook Uploader ] HTTP 403 Forbidden - Passwords not matching given: %s vs required: %s', $valid_upload, $password ), 'error' );
            header( "HTTP/1.0 403 Forbidden" );
            die();
        }

        $settings = Helper::load_all_settings( $settings_id );

        if ( ! $settings ) {
            Helper::log( sprintf( '[ Webhook Uploader ] HTTP 404 Not Found - Settings for ID %d not found', $settings_id ), 'error' );
            header( "HTTP/1.0 404 Not Found" );
            die();
        }

        $setting = reset( $settings );

        if ( $setting[ 'is_webhook_upload' ] === false ) {
            Helper::log( sprintf( '[ Webhook Uploader ] HTTP 403 Forbidden - HTTP Upload disabled' ), 'error' );
            header( "HTTP/1.0 403 Forbidden" );
            die();
        }

        if ( ! $_FILES ) {

            Helper::log( sprintf( '[ Webhook Uploader ] HTTP 400 Bad Request - Missing Files in POST' ), 'error' );

            header( "HTTP/1.0 400 Bad Request" );
            echo 'FileUpload must be via POST and must contain at least one file!';
            die();
        }

        if ( ! array_key_exists( 'fileToUpload', $_FILES ) ) {
            Helper::log( sprintf( '[ Webhook Uploader ] HTTP 406 Not Acceptable - File Identifier not "fileToUpload" %s', json_encode( $_FILES ) ), 'error' );

            header( "HTTP/1.0 406 Not Acceptable" );
            echo 'File must be sent with identifier fileToUpload!';
            die();
        }

        $uploaded_file = $_FILES[ 'fileToUpload' ];

        if ( ! array_key_exists( 'error', $uploaded_file ) ) {
            Helper::log( sprintf( '[ Webhook Uploader ] HTTP 400 Bad Request - Files array does not contain error field: %s', json_encode( $_FILES ) ), 'error' );
            header( "HTTP/1.0 400 Bad Request" );
            echo 'Could not verify POST Payload!';
            die();
        }

        if ( $_FILES[ 'fileToUpload' ][ 'error' ] !== UPLOAD_ERR_OK ) {
            Helper::log( sprintf( '[ Webhook Uploader ] HTTP 400 Bad Request - Files Upload error occured: %s', json_encode( $_FILES ) ), 'error' );
            header( "HTTP/1.0 400 Bad Request" );
            echo 'Could not verify POST Payload!';

            Helper::log( '[ Webhook Uploader ] Error during Webhook File upload occured!', 'error' );

            die();
        }

        Helper::log( sprintf( '[ Webhook Uploader ] Received valid POST Request to add File %s to Configuration %s...', $uploaded_file[ 'name' ], $setting[ 'settings-name' ] ) );

        $allowed_payload = [
            'id'        => '',
            'name'      => '',
            'model'     => '',
            'username'  => '',
            'user_id'   => '',
            'request'   => '',
            'time'      => '',
            'hash'      => '',
            'filename'  => '',
            'md5'       => '',
            'key'       => '',
            'skip_scan' => '',
        ];

        foreach ( $allowed_payload as $arg => $val ) {

            if ( ! array_key_exists( $arg, $_POST ) ) {
                continue;
            }

            $allowed_payload[ $arg ] = sanitize_text_field( $_POST[ $arg ] );
        }

        $is_breeze = false;

        $is_windows = $this->check_windows_upload( $allowed_payload, $setting );

        if ( ! $is_windows ) {
            $is_breeze = $this->check_breeze_upload( $allowed_payload, $setting );
        }

        if ( ! $is_windows && ! $is_breeze ) {
            die();
        }

        if ( $allowed_payload[ 'skip_scan' ] === '' ) {
            $this->file_management->scan_upload_folder( $settings_id );
        }

        exit;
    }

    private function is_breeze_uploader( $payload ) {

        $ref = strtolower( $payload[ 'id' ] );

        return strpos( $ref, 'breeze' ) !== false;
    }

    private function check_windows_upload( $allowed_payload, $setting ) {


        if ( $this->is_breeze_uploader( $allowed_payload ) ) {
            return false;
        }

        $uploaded_file = $_FILES[ 'fileToUpload' ];

        $target_directory = Helper::ensureTrailingBackslash( WP_CONTENT_DIR ) . $setting[ 'media_folder' ];

        $target_directory = str_replace( '//', '/', $target_directory );

        if ( ! strlen( $allowed_payload[ 'filename' ] ) ) {

            header( "HTTP/1.0 400 Bad Request" );
            echo 'Filename missing in payload!';

            Helper::log( sprintf( '[ Webhook Uploader - Windows ] HTTP 400 Bad Request - Filename missing in Payload! %s', json_encode( $allowed_payload ) ), 'error' );

            return false;
        }


        $filename = basename( $allowed_payload[ 'filename' ] );

        Helper::log( sprintf( '[ Webhook Uploader - Windows ] Moving File %s to target folder %s (new filename: %s)', $uploaded_file[ 'tmp_name' ], $target_directory, $filename ) );

        $target_file = Helper::ensureTrailingBackslash( $target_directory ) . $filename;

        if ( ! move_uploaded_file( $uploaded_file[ 'tmp_name' ], $target_file ) ) {
            Helper::log( sprintf( '[ Webhook Uploader - Windows ] Could not move file %s to target folder %s!', $uploaded_file[ 'tmp_name' ], $target_directory ), 'error' );
            header( "HTTP/1.0 400 Bad Request" );
            echo 'Could not move file to target directory!';

            return false;
        }

        Helper::log( sprintf( '[ Webhook Uploader - Windows ] Successfully moved file %s to target folder %s. Used configuration: %s', $uploaded_file[ 'tmp_name' ], $target_directory, json_encode( $allowed_payload ) ) );

        header( 'HTTP/1.0 201 Created' );

        return true;
    }

    private function check_breeze_ping( $allowed_payload, $setting ) {

        if ( $allowed_payload[ 'request' ] !== 'get_status' ) {

            return false;
        }

        $valid_upload = get_query_var( 'pg-upload', false );
        $checksum     = sha1( "breeze" . $allowed_payload[ 'id' ] . $valid_upload . $allowed_payload[ 'filename' ] . $allowed_payload[ 'md5' ] );

        if ( $checksum !== $allowed_payload[ 'key' ] ) {
            header( "HTTP/1.0 403 Forbidden" );

            Helper::log( '[ Webhook Uploader - Breeze Ping ] Checksum Error occured, did you forget to add your Webhook upload password to the breeze uploader?', 'error' );

            return false;
        }


    }

    private function check_breeze_upload( $allowed_payload, $setting ) {

        $valid_upload  = get_query_var( 'pg-upload', false );

        if ( strpos( $valid_upload, '-' ) !== false ) {

            list( $valid_upload, ) = explode( '-', $valid_upload );
        }

        $uploaded_file = $_FILES[ 'fileToUpload' ];
        $checksum      = sha1( "breeze" . $allowed_payload[ 'id' ] . $valid_upload . $allowed_payload[ 'filename' ] . $allowed_payload[ 'md5' ] );

        if ( $checksum !== $allowed_payload[ 'key' ] ) {
            header( "HTTP/1.0 403 Forbidden" );

            Helper::log( '[ Webhook Uploader - Breeze ] Checksum Error occured, did you forget to add your Webhook upload password to the breeze uploader?', 'error' );

            return false;
        }

        if ( $allowed_payload[ 'request' ] !== 'upload' ) {

            header( "HTTP/1.0 400 Bad Request" );
            echo 'Could not verify POST Payload!';

            Helper::log( sprintf( '[ Webhook Uploader - Breeze ] HTTP 400 Bad Request - Could not verifiy POST Payload %s', json_encode( $allowed_payload ) ), 'error' );

            return false;
        }


        if ( strcasecmp( md5_file( $uploaded_file[ 'tmp_name' ] ), $allowed_payload[ 'md5' ] ) != 0 ) {
            header( "HTTP/1.0 400 Bad Request" );
            echo 'Could not verify POST Payload!';

            Helper::log( '[ Webhook Uploader - Breeze ] File checksum mismatch!', 'error' );

            return false;
        }

        $target_directory = Helper::ensureTrailingBackslash( WP_CONTENT_DIR ) . $setting[ 'media_folder' ];
        $target_directory = str_replace( '//', '/', $target_directory );

        $filename = basename( $allowed_payload[ 'filename' ] );
        $ext      = pathinfo( $filename, PATHINFO_EXTENSION );
        $pattern  = '-%s.%s';

        $filename = str_replace( '.' . $ext, sprintf( $pattern, $this->generate_suffix( 3 ), $ext ), $filename );

        Helper::log( sprintf( '[ Webhook Uploader - Breeze ] Moving File %s to target folder %s (new filename: %s)', $uploaded_file[ 'tmp_name' ], $target_directory, $filename ) );

        $target_file = Helper::ensureTrailingBackslash( $target_directory ) . $filename;

        if ( ! move_uploaded_file( $uploaded_file[ 'tmp_name' ], $target_file ) ) {
            Helper::log( sprintf( '[ Webhook Uploader - Breeze ] Could not move file %s to target folder %s!', $uploaded_file[ 'tmp_name' ], $target_directory ), 'error' );
            header( "HTTP/1.0 400 Bad Request" );
            echo 'Could not move file to target directory!';

            return false;
        }

        Helper::log( sprintf( '[ Webhook Uploader - Breeze ] Successfully moved file %s to target folder %s. Used configuration: %s', $uploaded_file[ 'tmp_name' ], $target_directory, json_encode( $allowed_payload ) ) );

        header( 'HTTP/1.0 200 OK' );
        echo 'File succesfully transfered!';
        return true;
    }

    private function generate_suffix( $length = 3 ) {

        $suffix = [];

        while ( sizeof( $suffix ) < $length ) {
            $suffix[] = chr( mt_rand( 65, 90 ) );
        }

        return implode( '', $suffix );
    }
}
