<?php
/**
 * Created by PhpStorm.
 * User: b2wsra
 * Date: 2019-03-18
 * Time: 08:48
 */

namespace Photobooth;


use Photobooth\Statistics\XMLStatistics;

class Migration {

    /**
     * @var string
     */
    private $current_version;

    /**
     * @var array
     */
    private $migrations;

    /**
     * Migration constructor.
     *
     * @param $version
     * @param bool $force_migration
     */
    public function __construct( $version, $force_migration = false ) {

        $this->current_version = $version;

        if ( $force_migration ) {
            delete_option( 'photobooth-migrations' );
        }

        $migrated_version = get_option( 'photobooth-migrations' );

        if ( $migrated_version === $this->current_version ) {
            return;
        }

        $this->migrations = [];

        add_action( 'admin_init', [ $this, 'run_migration' ], 1 );

        $this->enable_statistics_security();
        $this->enable_http_post_upload_in_all_configs_and_set_random_http_password();
        $this->fix_statistic_xml_import_bug();
    }


    function run_migration() {

        foreach ( $this->migrations as $migration ) {

            list( $version, $closure, $version_compare ) = array_values( $migration );

            if ( ! version_compare( $version, $this->current_version, $version_compare ) ) {
                continue;
            }

            call_user_func( $closure );

            Helper::log( sprintf( '[ Photobooth Migrations ] Successfully executed migration from version %s to %s (%s)...', $this->current_version, $version, $version_compare ) );
        }

        update_option( 'photobooth-migrations', $this->current_version );
    }

    public function add_migration( $version, \Closure $closure, $version_compare = '>=' ) {

        array_push( $this->migrations, compact( 'version', 'closure', 'version_compare' ) );
    }


    private function enable_statistics_security() {

        $this->add_migration( '3.1.1',
            function() {

                $settings = [
                    'pg_is_statistic_secured' => true,
                    'pg_statistic_password'   => uniqid(),
                ];

                foreach ( $settings as $setting => $value ) {

                    if ( get_field( $setting, 'option' ) === $value ) {
                        continue;
                    }

                    update_field( $setting, $value, 'option' );

                }

            } );
    }

    private function enable_http_post_upload_in_all_configs_and_set_random_http_password() {

        $this->add_migration( '3.1.1',
            function() {

                $configs = Helper::load_all_settings();

                foreach ( $configs as $config ) {
                    update_field( 'is_http_post_upload_enabled', true, $config[ 'settings-post-id' ] );
                }

                if ( get_field( 'pg_webhook_upload_password', 'options' ) ) {
                    return;
                }

                update_field( 'pg_webhook_upload_password', uniqid(), 'options' );

            } );
    }

    private function fix_statistic_xml_import_bug() {

        $this->add_migration( '3.1.6',
            function() {

                global $wpdb;

                $query = sprintf( "UPDATE %s SET c_output = c_file, c_photos = 1 WHERE c_output = ''", XMLStatistics::table() );

                $wpdb->query( $query );

            } );
    }
}
