<?php declare(strict_types=1);

namespace Photobooth\Plugins;

use WP_Post;
use WP_Theme;

class SmartTVMixin
{
    public function init()
    {
        add_filter('page_template', fn(string $page_template) => $this->addPageTemplate($page_template));
        add_filter(
            'theme_page_templates',
            fn(
                array $post_templates,
                WP_Theme $wp_theme,
                ?WP_Post $post,
                string $post_type
            ) => $this->addPageTemplateDescription($post_templates, $wp_theme, $post, $post_type),
            10,
            4
        );

        add_filter(
            'photobooth/render_code_content/query_args',
            fn(array $query, array $settings, string $the_code, string $type) => $this->modifyQueryArgs(
                $query,
                $settings,
                $the_code,
                $type
            ),
            10,
            4
        );

        add_filter(
            'photobooth/render_code_content/twig_template',
            fn(string $template, array $data, array $settings, string $type) => $this->handleTvTemplate(
                $template,
                $data,
                $settings,
                $type
            ),
            10,
            4
        );

        add_filter(
            'photobooth/render_code_content/enable_ip_ban',
            fn(bool $banUser, array $settings, string $type) => $this->disableIpBan($banUser, $settings, $type),
            10,
            3
        );
    }


    private function addPageTemplate(string $page_template): string
    {
        if (get_page_template_slug() !== 'smart-tv-template.php') {
            return $page_template;
        }

        return PG_TEMPLATES_DIR.'/page/smart-tv-template.php';
    }

    private function addPageTemplateDescription(
        array $post_templates,
        WP_Theme $wp_theme,
        ?WP_Post $post,
        string $post_type
    ): array {

        if ($post_type !== 'page') {
            return $post_templates;
        }

        $post_templates['smart-tv-template.php'] = __('[Photobooth Gallery] Smart TV Template', 'photobooth-gallery');

        return $post_templates;
    }

    /**
     * Change Query to be ordered DESCENDING leaving the newest images always first
     *
     * @param array $query
     * @param array $settings
     * @param string $the_code
     * @param string $type
     * @return array
     */
    private function modifyQueryArgs(array $query, array $settings, string $the_code, string $type): array
    {
        if ($type !== 'smart-tv') {
            return $query;
        }

        $query['order'] = 'DESC';

        return $query;
    }

    /**
     * Add new template for SmartTVs
     *
     * @param string $template
     * @param array $data
     * @param array $settings
     * @param string $type
     * @return string
     */
    private function handleTvTemplate(string $template, array $data, array $settings, string $type): string
    {

        if ($type !== 'smart-tv') {
            return $template;
        }

        return 'photobooth-portfolio-smart-tv';
    }

    /**
     * Disable IP Ban for SmartTV Requests
     * @param bool $banUser
     * @param array $settings
     * @param string $type
     * @return bool
     */
    private function disableIpBan(bool $banUser, array $settings, string $type): bool
    {

        if ($type !== 'smart-tv') {
            return $banUser;
        }

        return false;
    }
}
