<?php
/**
 * Created by PhpStorm.
 * User: b2wsra
 * Date: 15.08.2018
 * Time: 12:54
 */

namespace Photobooth;

use Photobooth\GuzzleHttp\Client;
use Photobooth\GuzzleHttp\Exception\RequestException;
use Photobooth\PedroBorges\MetaTags\MetaTags;
use Photobooth\Psr\Http\Message\ResponseInterface;
use Photobooth\Ramsey\Uuid\Uuid;
use Photobooth\StringTemplate\Engine;
use WP_Post;
use WP_Query;

define('PHOTOBOOTH_GALLERY_DB_VERSION_SOCIAL_MEDIA', '1.0.8');
define('PHOTOBOOTH_IPSTACK_API_KEY', '28be4a19703fe5b77fd8add13afac118');

class SocialMedia
{


    const SHARE = 'SHARE';
    const VIEW = 'VIEW';

    private array $settings;

    private array $media;

    private array|bool $shares;

    private int $result_size;

    private FileManagement $file_manager;

    private bool $skipShareScan  = false;

    public function setSkipShareScan(bool $skipShareScan): void
    {
        $this->skipShareScan = $skipShareScan;
    }

    private static ?SocialMedia $instance = null;

    public static function getinstance(array $settings = []): SocialMedia {

        if (self::$instance === null) {
            self::$instance = new self();
        }

        if (count($settings) > 0) {
            self::$instance->updateSettings($settings);
        }

        return self::$instance;
    }

    public function updateSettings(array $settings): void {
        $this->settings = $settings;

        $media = json_decode(
            file_get_contents(
                apply_filters('pb/social_media/folder', PG_BASE_DIR.'/templates').'/'.'social_media.json'
            ),
            true
        );

        if (!$media) {
            $media = [];
        }

        if (!$this->settings['social_media_buttons']) {
            $this->settings['social_media_buttons'] = [];
        }

        $keys = array_values($this->settings['social_media_buttons']);

        $this->media = array_filter(
            $media,
            function ($network) use ($keys) {

                return in_array($network, $keys);
            },
            ARRAY_FILTER_USE_KEY
        );

        $this->media = array_merge(array_flip($keys), $this->media);
    }

    private function __construct()
    {
        $this->shares = false;
        $this->file_manager = new FileManagement();
    }

    public static function init()
    {

        $installed_ver = get_option('PHOTOBOOTH_GALLERY_DB_VERSION_SOCIAL_MEDIA');

        if ($installed_ver === PHOTOBOOTH_GALLERY_DB_VERSION_SOCIAL_MEDIA) {
            return;
        }

        $table_name = self::table();

        $sql = "CREATE TABLE $table_name (
                id mediumint(9) NOT NULL AUTO_INCREMENT,
                `c_uuid` VARCHAR(64) NOT NULL,
                `c_reference` INT NOT NULL,
                `is_shareable` TINYINT(1) NOT NULL,
                `c_timestamp` TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
                `c_referrer` MEDIUMTEXT,
                `c_user_agent` MEDIUMTEXT,
                `c_network` VARCHAR(255) NOT NULL,
                `c_code` VARCHAR(255) NOT NULL,
                `c_config` VARCHAR(255) NOT NULL,
                `c_mime_type` VARCHAR(255) NOT NULL,
                `c_target` MEDIUMTEXT NOT NULL,
                `c_file` MEDIUMTEXT NOT NULL,
                `c_type` VARCHAR(16) NOT NULL,
                `c_encoded_geo_data` MEDIUMTEXT,
                `c_geo_country_code` VARCHAR(10),
                `c_geo_city` VARCHAR(255),
                `c_geo_zip` VARCHAR(10),
                `c_geo_lat` DECIMAL(10, 8),
                `c_geo_long` DECIMAL(10, 8),
                INDEX `network` (`c_network`),
                INDEX `uuid` (`c_uuid`),
                INDEX `reference` (`c_reference`),
                INDEX `code` (`c_code`),
                INDEX `mime` (`c_mime_type`),
                INDEX `config` (`c_config`),
                INDEX `type` (`c_type`),
                INDEX `zip` (`c_geo_zip`),
                INDEX `city` (`c_geo_city`),
                PRIMARY KEY  (id)
        );";

        require_once(ABSPATH.'wp-admin/includes/upgrade.php');

        dbDelta($sql);

        update_option('PHOTOBOOTH_GALLERY_DB_VERSION_SOCIAL_MEDIA', PHOTOBOOTH_GALLERY_DB_VERSION_SOCIAL_MEDIA);
    }

    public function get_social_media($item)
    {

        $buttons = [];

        $item = array_merge($this->settings, $item);
        $share = $item['hide_download_all_button'] ?? false;

        foreach ($this->media as $type => $media) {

            // Do not show download all if item list is smaller than hide_download_all
            if ($type === 'download-all' && $share && $this->result_size < $item['hide_download_all']) {
                Helper::log(
                    sprintf(
                        '[ Social Media ] Download All Button disabled for this view, because %d media files will be shown!',
                        $this->result_size
                    )
                );
                continue;
            }

            $uuid = Uuid::uuid4()
                ->toString();

            set_transient($uuid, $item['payload'], 30 * MINUTE_IN_SECONDS);

            $media['share_type'] = $type;
            $media['payload'] = $uuid;
            $media['target'] = $item['full'];
            $media['title'] = __($media['title'], 'photobooth-gallery');

            $buttons[] = $media;
        }

        return $buttons;
    }

    public function log($payload)
    {

        if (is_user_logged_in()) {
            return -1;
        }

        $filename = 'undefined.ext';
        $attachment_id = 0;
        $attachment_url = site_url();
        $mime = 'N/A';
        $type = self::VIEW;

        switch ($payload['action']) {
            case 'admin-remove-images' :
            case 'pagination-link-clicked' :
            case 'new-code-button-clicked' :
            case 'share':
                return 0;
            case 'download' :
                $attachment_id = $payload['additional_data']['id'];
                $filename = $payload['additional_data']['filename'];
                $mime = $payload['additional_data']['mime'];
                $type = self::SHARE;
                break;
            case 'image-preview-clicked' :
                $attachment_url = $payload['additional_data'];
                $filename = $this->wp_basename($attachment_url);
                $attachment_id = $this->file_manager->get_image_from_url($filename);
                $filename = get_post_meta($attachment_id, 'real_filename', true);
                $mime = get_post_mime_type($attachment_id);
                break;
            case 'video-preview-clicked' :
                $attachment_url = $payload['additional_data']['src'];
                $attachment_id = $this->file_manager->get_image_from_url($attachment_url);
                $filename = get_post_meta($attachment_id, 'real_filename', true);
                $mime = $payload['additional_data']['type'];
                break;
        }

        $data = [
            'c_uuid' => hash('sha256', $filename.$payload['action']),
            'c_reference' => $attachment_id,
            'is_shareable' => 0,
            'c_network' => $payload['action'],
            'c_code' => $payload['code'],
            'c_config' => $payload['config'],
            'c_target' => $attachment_url,
            'c_file' => $filename,
            'c_mime_type' => $mime,
            'c_type' => $type,
            'c_referrer' => $_SERVER['HTTP_REFERER'] ?? __('not-available', 'photobooth-gallery'),
            'c_user_agent' => $_SERVER['HTTP_USER_AGENT'] ?? __('not-available', 'photobooth-gallery'),
        ];

        $definition = [
            '%s',
            '%d',
            '%d',
            '%s',
            '%s',
            '%s',
            '%s',
            '%s',
            '%s',
            '%s',
            '%s',
            '%s',
        ];

        $id = $this->insert_row($data, $definition);

        if ($id === false) {
            return false;
        }

        $this->update_ip_data($id);

        return $id;
    }

    private function wp_basename($url)
    {

        $name = basename($url);

        if (strpos($url, '-') === false) {
            return $name;
        }

        if (strpos($url, 'x') === false) {
            return $name;
        }

        $ext = pathinfo($name, PATHINFO_EXTENSION);
        $names = explode('-', $name);
        $name = reset($names);

        return sprintf('%s.%s', $name, $ext);
    }

    private function insert_row($data, $definition)
    {

        global $wpdb;

        $table_name = $this->get_table();

        $result = $wpdb->insert($table_name, $data, $definition);

        if (!$result) {
            Helper::log(
                sprintf('[ Social Media ] Could not INSERT %s into %s', json_encode($data), $table_name),
                'error'
            );

            return false;
        }

        return $wpdb->insert_id;
    }

    private function get_table()
    {

        return self::table();
    }

    public static function table()
    {

        global $wpdb;

        $table_name = $wpdb->prefix.'pg_social_media_views';

        return $table_name;
    }

    private function update_ip_data($document_id)
    {

        $gather_info = get_field('pg_is_extended_geo_data', 'option');

        if (!$gather_info) {
            return;
        }

        $pb_api_key = get_field('pg_ip_stack_api_key', 'option');

        if (!$pb_api_key) {
            $pb_api_key = PHOTOBOOTH_IPSTACK_API_KEY;
        }

        if (!$document_id) {
            return;
        }

        $ip_address = Helper::get_client_ip();

        if (!$ip_address) {
            return;
        }

        $client = new Client(['base_uri' => 'http://api.ipstack.com/']);

        $promise = $client->requestAsync(
            'GET',
            urlencode($ip_address),
            [
                'query' => [
                    'access_key' => $pb_api_key,
                ],
            ]
        );

        $promise->then(function (ResponseInterface $response) use ($document_id) {

            if ($response->getStatusCode() !== 200) {
                return;
            }

            $ip_data = json_decode(
                $response->getBody()
                    ->getContents(),
                true
            );

            if (!$ip_data) {
                return;
            }

            unset($ip_data['ip']);

            $payload = [
                'c_encoded_geo_data' => json_encode($ip_data),
                'c_geo_country_code' => $ip_data['country_code'] ?? '-',
                'c_geo_city' => $ip_data['city'] ?? '-',
                'c_geo_zip' => $ip_data['zip'] ?? '-',
                'c_geo_lat' => $ip_data['latitude'] ?? 0.0,
                'c_geo_long' => $ip_data['longitude'] ?? 0.0,
            ];

            $definition = [
                '%s',
                '%s',
                '%s',
                '%s',
                '%f',
                '%f',
            ];

            $this->update_row($document_id, $payload, $definition);

        },
            function (RequestException $e) use ($pb_api_key, $ip_address) {

                Helper::log(
                    sprintf(
                        '[ Get Geo IP Data] Error while contacting webservice! %s Consider using another api key! [ %s, %s ]',
                        $e->getMessage(),
                        $pb_api_key,
                        $ip_address
                    ),
                    'error'
                );

            })
            ->wait();

    }

    private function update_row($id, $data, $definition)
    {

        global $wpdb;

        $table_name = $this->get_table();

        $where = [
            'id' => $id,
        ];

        return $wpdb->update($table_name, $data, $where, $definition);
    }

    public function render_social_tag_file($file)
    {

        $tags = new MetaTags();

        $url = Helper::abs_path_to_url($file);
        $type = 'image/jpeg';

        $ext = strtolower(pathinfo($file, PATHINFO_EXTENSION));

        if ($ext === 'gif') {
            $type = 'image/gif';
        }

        if ($ext === 'mp4') {
            $type = 'video/mp4';
        }

        $tags->og('url', $url);

        $tags->twitter('card', 'summary');

        if (strpos($type, 'image') !== false) {
            $tags->og('type', 'image.gallery');
            $tags->og('image', $url);
        }

        if (strpos($type, 'video') !== false) {
            $tags->og('type', 'video.other');
            $tags->og('video', $url);
            $tags->og('video:type', $type);
        }

        return $tags->render();
    }

    public function render_social_tags($hash = false)
    {

        $hash = $hash ?: get_query_var('pg-payload');

        if (!$hash) {
            return false;
        }

        $payload = self::decode($hash);

        if (!$payload) {
            return false;
        }

        $settings = Helper::load_setting($payload['settings']);

        if (!$settings) {
            return false;
        }

        if (!$settings['is_social_media']) {
            return false;
        }

        $attachment_id = $payload['the-id'] ?? false;

        if (!$attachment_id) {

            $attachment = $this->get_first_image($payload, $settings);
            $attachment_id = $attachment->ID ?? false;
        }

        if (!$attachment_id) {
            return false;
        }


        $type = get_post_mime_type($attachment_id);
        $url = wp_get_attachment_url($attachment_id);

        if (!$type) {
            $type = 'image/jpeg';
        }

        $tags = new MetaTags();

        $tags->og('title', $settings['social_media_subject']);
        $tags->og('description', $settings['social_media_description']);

        $parts = [
            Helper::removeTrailingBackslash($settings['social_media_page']),
            'pg-payload',
            $hash,
        ];

        $tags->og('url', Helper::ensureTrailingBackslash(implode('/', $parts)));

        $tags->twitter('card', 'summary');

        if (strpos($type, 'image') !== false) {
            $tags->og('type', 'image.gallery');
            $tags->og('image', $url);
        }

        if (strpos($type, 'video') !== false) {
            $tags->og('type', 'video.other');
            $tags->og('video', $url);
            $tags->og('video:type', $type);
        }

        return $tags->render();
    }

    public static function decode($payload)
    {

        if (!$payload) {
            return false;
        }

        // New Method works with database Lookup
        if (strlen($payload) === 64) {

            $social = SocialMedia::getinstance();
            $document = $social->get_document_from_payload($payload);

            if (!$document) {
                return false;
            }

            $datablock = [
                'the-code' => $document->c_code,
                'settings' => $document->c_config,
                'the-id' => $document->c_reference,
            ];

            Helper::log(sprintf('[ Decoder ] Decoded payload <%s> to %s...', $payload, json_encode($datablock)));

            return $datablock;
        }

        $test = get_transient($payload);

        if ($test) {
            $payload = $test;
        }

        $decrypted = Helper::decrypt($payload);

        if (!$decrypted) {
            return false;
        }

        $datablock = json_decode($decrypted, true);

        if (!$datablock) {
            return false;
        }

        Helper::log(sprintf('[ Decoder Legacy Code! ] Decoded payload <%s> to %s...', $payload, $decrypted));

        return $datablock;
    }

    private function get_document_from_payload($hash = false)
    {

        $hash = $hash ?: get_query_var('pg-payload');

        // Legacy code, payload is not exactly 64 bytes long, which is the sha256 code
        if (strlen($hash) !== 64) {
            return false;
        }

        $hash = sanitize_text_field($hash);

        $document = $this->load_rows($hash, self::SHARE, 1);

        if (!$document) {
            Helper::log(
                sprintf('[ View Shared Media ] Could not find any media object with the hash <%s>', $hash),
                'warning'
            );

            return false;
        }

        return $document;
    }

    private function load_rows($hash, $type = self::VIEW, $limit = 0)
    {

        global $wpdb;

        $table_name = $this->get_table();
        $hash = esc_sql($hash);
        $limit = intval($limit);

        $query = "SELECT * FROM {$table_name} WHERE c_uuid = '{$hash}' AND c_type = '{$type}' ORDER BY c_timestamp DESC";

        if ($limit > 0) {
            $query .= ' LIMIT '.$limit;
        }

        $results = $wpdb->get_results($query);

        if ($limit === 1 && $results) {
            return reset($results);
        }

        return $results;
    }

    private function get_first_image($atts, $settings)
    {

        $the_code = $atts['the-code'] ?? false;

        if (!$settings) {
            return false;
        }

        if (!$the_code) {
            return false;
        }

        $the_code = sanitize_text_field($the_code);

        if (!is_string($the_code)) {

            return false;
        }

        $query = new WP_Query([
            'post_type' => 'attachment',
            'post_status' => 'any',
            'posts_per_page' => 1,
            'orderby' => 'meta_value_num',
            'meta_key' => 'file_date',
            'order' => 'DESC',
            'meta_query' => [
                [
                    'key' => 'real_filename',
                    'value' => $the_code,
                    'compare' => 'LIKE',
                ],
            ],
        ]);

        if (!$query->have_posts()) {
            return false;
        }

        $posts = $query->get_posts();

        return reset($posts);
    }

    public function share($payload)
    {

        $attachment_id = $payload['the-id'] ?? false;

        if (!$attachment_id) {
            return false;
        }

        $attachment_id = intval($attachment_id);
        $filename = get_post_meta($attachment_id, 'real_filename', true);
        $attachment_url = wp_get_attachment_url($attachment_id);
        $network = sanitize_text_field($payload['network']);

        if (!$filename) {
            return false;
        }

        if (!$network) {
            return false;
        }

        $media = $this->media[$network];

        $mime = get_post_mime_type($attachment_id);

        $data = [
            'c_uuid' => hash('sha256', $filename.$payload['network']),
            'c_reference' => $attachment_id,
            'is_shareable' => $media['is_shareable'] ?? 0,
            'c_network' => $payload['network'],
            'c_code' => $payload['the-code'],
            'c_config' => $payload['settings'],
            'c_target' => $attachment_url,
            'c_file' => $filename,
            'c_mime_type' => $mime,
            'c_type' => self::SHARE,
            'c_referrer' => $_SERVER['HTTP_REFERER'] ?? __('not-available', 'photobooth-gallery'),
            'c_user_agent' => $_SERVER['HTTP_USER_AGENT'] ?? __('not-available', 'photobooth-gallery'),
        ];

        $definition = [
            '%s',
            '%d',
            '%d',
            '%s',
            '%s',
            '%s',
            '%s',
            '%s',
            '%s',
            '%s',
            '%s',
            '%s',
        ];

        $id = $this->insert_row($data, $definition);

        if ($id === false) {
            return false;
        }

        $this->update_ip_data($id);

        $engine = new Engine();

        $media = $this->media[$network];

        if (strpos($mime, 'image') !== false) {

            $preview = wp_get_attachment_image_src($attachment_id, 'pg-preview');
            $media['preview'] = $preview[0];

            if (!$preview) {
                return false;
            }
        }

        $filename = explode('/', $attachment_url);
        $filename = end($filename);

        $download_info = [
            'code' => $data['c_code'],
            'file' => get_attached_file($attachment_id),
        ];

        $media['mime'] = $mime;
        $media['subject'] = rawurlencode($this->settings['social_media_subject']);
        $media['title'] = rawurlencode(__($media['title'], 'photobooth-gallery'));
        $media['share_url'] = $this->get_share_url($data['c_uuid']);
        $media['download'] = $this->get_download_url($download_info);
        $media['redirect'] = $engine->render($media['target'], $media);
        $media['filename'] = $filename;

        if (array_key_exists('applink', $media)) {
            $media['applink'] = $engine->render($media['applink'], $media);
        }

        return $media;
    }

    private function get_share_url($payload)
    {

        $parts = [
            Helper::removeTrailingBackslash($this->settings['social_media_page']),
            'pg-payload',
            $payload,
        ];

        $url = implode('/', $parts);

        return $url;
    }

    private function get_download_url($payload)
    {

        $parts = [
            site_url(),
            'pg-download',
            base64_encode(json_encode($payload)),
        ];

        $url = implode('/', $parts);

        return $url;
    }

    public function is_deleteable($post)
    {

        if ($this->share_count($post) === 0) {
            return true;
        }

        $post_id = 0;

        if ($post instanceof WP_Post) {
            $post_id = $post->ID;
        }

        if (is_int($post)) {
            $post_id = $post;
        }

        if ($post_id === 0) {
            return 0;
        }

        $last_view = $this->get_share_data($post_id, self::VIEW, 1);

        if (!$last_view) {
            return true;
        }

        $max_age = get_field('pg_max_social_share', 'options');

        if (!$max_age) {
            $max_age = '30_days';
        }

        $max_age = 'P'.str_replace('_days', 'D', $max_age);

        $date = new \DateTime($last_view->c_timestamp);
        $date->add(new \DateInterval($max_age));

        return $date->format(__('Y-m-d H:i', 'photobooth-gallery'));
    }

    public function share_count($post)
    {

        if ($this->skipShareScan === true) {
            return 0;
        }

        $post_id = 0;

        if ($post instanceof WP_Post) {
            $post_id = $post->ID;
        }

        if (is_int($post)) {
            $post_id = $post;
        }

        if ($post_id === 0) {
            return 0;
        }

        if ($this->shares === false) {
            $this->init_shares();
        }

        return $this->shares[$post_id] ?? 0;
    }

    /**
     * Count all currently shared images and group them by Post ID
     *
     * @return bool
     */
    private function init_shares()
    {

        global $wpdb;

        $this->shares = [];

        $table_name = $this->get_table();
        $type = self::SHARE;

        $max_age = get_field('pg_max_social_share', 'option');

        if (!$max_age) {
            $max_age = '30days';
        }

        $max_age = date('Y-m-d H:i:s', strtotime('-'.str_replace('_', '', $max_age)));

        $query = "SELECT c_reference as post_id, count(*) as views FROM {$table_name} WHERE c_type = '{$type}' AND c_timestamp > '{$max_age}' GROUP BY c_reference";

        $results = $wpdb->get_results($query);

        foreach ($results as $result) {
            $this->shares[intval($result->post_id)] = intval($result->views);
        }

        return true;
    }

    private function get_share_data($attachment_id, $type = self::VIEW, $limit = 0)
    {

        global $wpdb;

        $table_name = $this->get_table();
        $attachment_id = esc_sql($attachment_id);
        $limit = intval($limit);

        $query = "SELECT * FROM {$table_name} WHERE c_reference = {$attachment_id} AND c_type = '{$type}' ORDER BY c_timestamp DESC";

        if ($limit > 0) {
            $query .= ' LIMIT '.$limit;
        }

        $results = $wpdb->get_results($query);

        if ($limit === 1 && $results) {
            return reset($results);
        }

        return reset($results);
    }

    public function view()
    {

        if (!PG_DEBUG && is_user_logged_in()) {
            return false;
        }

        $document = $this->get_document_from_payload();

        if (!$document) {
            return false;
        }

        $data = [
            'c_uuid' => $document->c_uuid,
            'c_reference' => $document->c_reference,
            'is_shareable' => $document->is_shareable,
            'c_network' => $document->c_network,
            'c_code' => $document->c_code,
            'c_config' => $document->c_config,
            'c_target' => $document->c_target,
            'c_file' => $document->c_file,
            'c_mime_type' => $document->c_mime_type,
            'c_type' => self::VIEW,
            'c_referrer' => $_SERVER['HTTP_REFERER'] ?? __('not-available', 'photobooth-gallery'),
            'c_user_agent' => $_SERVER['HTTP_USER_AGENT'] ?? __('not-available', 'photobooth-gallery'),
        ];

        $definition = [
            '%s',
            '%d',
            '%d',
            '%s',
            '%s',
            '%s',
            '%s',
            '%s',
            '%s',
            '%s',
            '%s',
            '%s',
        ];

        $id = $this->insert_row($data, $definition);

        if ($id !== false) {
            $this->update_ip_data($id);
        }

        if (!$document->is_shareable) {

            $download_info = [
                'code' => $document->c_code,
                'file' => get_attached_file($document->c_reference),
                'hash' => get_query_var('pg-payload'),
            ];

            wp_redirect($this->get_download_url($download_info));
            die;
        }

        $settings = Helper::load_setting($document->c_config);

        if (!$settings) {
            return false;
        }

        if (!$settings['social_media_page']) {

            $download_info = [
                'code' => $document->c_code,
                'file' => get_attached_file($document->c_reference),
                'hash' => get_query_var('pg-payload'),
            ];

            wp_redirect($this->get_download_url($download_info));
            die;
        }

        if ($settings['social_media_share_type'] === 'selected_media') {

            $download_info = [
                'code' => $document->c_code,
                'file' => get_attached_file($document->c_reference),
                'hash' => get_query_var('pg-payload'),
            ];

            wp_redirect($this->get_download_url($download_info));
            die;
        }

        return true;
    }

    public function get_latest_social_stats_for($attachment_ids = [])
    {

        if (!$attachment_ids) {
            return false;
        }

        $id_set = implode(',', $attachment_ids);

        global $wpdb;

        $table_name = $this->get_table();
        $type = self::VIEW;

        $query = "SELECT t1.* FROM {$table_name} t1 
					LEFT JOIN  {$table_name} t2 ON t1.c_reference = t2.c_reference AND t1.c_timestamp < t2.c_timestamp 
					WHERE t1.c_reference IN ( {$id_set} ) AND t1.c_type = '{$type}' AND t2.c_timestamp IS NULL GROUP BY t1.c_reference";

        $results = $wpdb->get_results($query);

        $stats = [];
        foreach ($results as $result) {
            $stats[$result->c_reference] = $result;
        }

        return $stats;
    }

    public function setResultSize($result_size)
    {

        $this->result_size = $result_size;
    }

    public function delete(int $referenceId)
    {
        global $wpdb;

        $table_name = $this->get_table();

        $where = [
            'c_reference' => $referenceId,
        ];

        return $wpdb->delete($table_name, $where);
    }

    /**
     * this is just for our gulp-pot generator to find all translateable values from the json file
     */
    protected function i18n()
    {

        $transient = [
            __('Download now', 'photobooth-gallery'),
            __('Download all', 'photobooth-gallery'),
            __('Send an e-mail', 'photobooth-gallery'),
            __('Share with Twitter', 'photobooth-gallery'),
            __('Share with Facebook', 'photobooth-gallery'),
            __('Share with Google Plus', 'photobooth-gallery'),
            __('Share with Linked-In', 'photobooth-gallery'),
            __('Share with Pinterest', 'photobooth-gallery'),
            __('Share with Stumbleupon', 'photobooth-gallery'),
            __('Share with Whatsapp', 'photobooth-gallery'),
            __('Share with WeChat', 'photobooth-gallery'),
            __('Share with Xing', 'photobooth-gallery'),
        ];
    }

}
