<?php
/**
 * Created by PhpStorm.
 * User: b2wsra
 * Date: 15.08.2018
 * Time: 12:54
 */

namespace Photobooth\Statistics;

use Photobooth\Helper;
use Photobooth\Renderer;
use Photobooth\SocialMedia;
use WhichBrowser\Constants\DeviceType;
use WhichBrowser\Model\Device;
use WhichBrowser\Model\Os;
use WhichBrowser\Parser;

class SocialMediaStatistics {

	/**
	 * @var string
	 */
	private $code;


	public function getCode() {
		return $this->code;
	}

	public function setCode( $code ) {
		$this->code = $code;
	}


	public function wp_init() {
		add_shortcode( 'photobooth-statistics-sharing-from', [ $this, 'shortcode_sharing_from' ] );
		add_shortcode( 'photobooth-statistics-sharing-to', [ $this, 'shortcode_sharing_to' ] );
		add_shortcode( 'photobooth-statistics-event-from', [ $this, 'shortcode_event_from' ] );
		add_shortcode( 'photobooth-statistics-event-to', [ $this, 'shortcode_event_to' ] );
		add_shortcode( 'photobooth-statistics-event-duration', [ $this, 'shortcode_event_duration' ] );
		add_shortcode( 'photobooth-statistics-event-profiles', [ $this, 'shortcode_event_profiles' ] );
		add_shortcode( 'photobooth-statistics-event-prints', [ $this, 'shortcode_event_prints' ] );

		add_action( 'wp_ajax_pg-elementor-widget-data', [ $this, 'ajax_get_data' ] );
		add_action( 'wp_ajax_nopriv_pg-elementor-widget-data', [ $this, 'ajax_get_data' ] );

		add_action( 'wp_enqueue_scripts', [ $this, 'enqueue_csrf' ] );
	}

	private function handle_code( $atts ) {

		if ( ! array_key_exists( 'the-code', $atts ) ) {
			return $atts;
		}

		$code = $atts['the-code'];
		$code = urldecode( $code );

		if ( strpos( $code, '|' ) === false ) {
			return $atts;
		}

		$data = explode( '|', $code );

		$code             = array_shift( $data );
		$atts['the-code'] = $code;
		$this->code       = $code;

		$from = array_shift( $data );

		if ( $from ) {
			$timestamp = new \DateTime();
			$timestamp->setTimestamp( intval( $from ) );
			$atts['from'] = $timestamp->format( 'Y-m-d H:i:s' );
		}

		$to = array_shift( $data );

		if ( $to ) {
			$timestamp = new \DateTime();
			$timestamp->setTimestamp( intval( $to ) );
			$atts['to'] = $timestamp->format( 'Y-m-d H:i:s' );
		}

		return $atts;
	}


	private function shortcode_query( $atts, $query, $table, $code_field, $format_result = true ) {

		$atts = $this->handle_code( $atts );
		$time = $this->get_time( $atts );

		$where = [];

		if ( $time ) {
			$where[] = $time;
		}

		if ( $atts['the-code'] ) {

			if ( $where ) {
				$where[] = 'AND';
			}

			$where[] = sprintf( "%s LIKE '%s'", $code_field, '%' . esc_sql( $atts['the-code'] ) . '%' );
		}


		if ( $where ) {
			array_unshift( $where, 'WHERE' );
		}

		$query = sprintf( $query, $table, implode( ' ', $where ) );

		if ( PG_LOG_VERBOSE ) {
			Helper::log( sprintf( '[ Statistic ] SQL Debug (shortcode): %s ', $query ) );
		}

		$result = $this->execute_query( $query );

		if ( ! is_array( $result ) ) {
			return '';
		}

		$result = reset( $result );

		if ( ! $result ) {
			return '';
		}

		if ( ! $format_result ) {
			return $result;
		}

        if (!$result->d_from) {
            return '';
        }

		$date = \DateTime::createFromFormat( 'Y-m-d H:i:s', $result->d_from );

		if ( ! $date ) {
			return '';
		}

		return $date->format( $atts['date-format'] );
	}

	function enqueue_csrf() {
		wp_localize_script( 'pg-elementor-statistics', 'pg_elementor_statistic', array(
			'ajaxurl' => admin_url( 'admin-ajax.php' ),
			'nonce'   => wp_create_nonce( 'pg-elementor-widget-data' ),
		) );
	}

	function ajax_get_data() {
		check_ajax_referer( 'pg-elementor-widget-data', 'csrf' );

		$this->code = sanitize_text_field( $_REQUEST['the-code'] ?? '' );

		$type = sanitize_text_field( $_REQUEST['type'] ?? '' );
		$view = sanitize_text_field( $_REQUEST['view'] ?? SocialMedia::VIEW );

		wp_send_json( [
			'data' => $this->count_by_network( $type, $view )
		] );
	}

	function shortcode_event_profiles( $atts ) {
		$atts = shortcode_atts( array(
			'class'    => '',
			'the-code' => get_query_var( 'pg-code' ),
			'c_folder' => 'Photobooth profile',
			'c_size'   => 'Number of sequences'
		), $atts, 'photobooth-statistics-event-profiles' );

		$atts = $this->handle_code( $atts );
		$time = $this->get_time( $atts );

		$where = [];

		if ( $time ) {
			$where[] = $time;
		}

		if ( $atts['the-code'] ) {

			if ( $where ) {
				$where[] = 'AND';
			}

			$where[] = sprintf( "c_output LIKE '%s'", '%' . esc_sql( $atts['the-code'] ) . '%' );
		}


		if ( $where ) {
			array_unshift( $where, 'WHERE' );
		}

		$table = XMLStatistics::table();

		$query = sprintf( 'SELECT COUNT(*) as c_size, c_images_folder as c_folder FROM %s %s GROUP BY c_images_folder ORDER BY c_folder ASC', $table, implode( ' ', $where ) );

		if ( PG_LOG_VERBOSE ) {
			Helper::log( sprintf( '[ Statistic ] SQL Debug (shortcode_event_profiles): %s ', $query ) );
		}

		$results = $this->execute_query( $query );

		if ( ! $results ) {
			return '';
		}

		$response = [];
		foreach ( $results as $result ) {
			$folder = explode( '\\', $result->c_folder );

			$folder = end( $folder );

			$result->c_folder = $folder;

			if ( array_key_exists( $folder, $response ) ) {
				$response[ $folder ]->c_size += $result->c_size;
				continue;
			}

			$response[ $folder ] = $result;
		}

		$results = array_values( $response );

		usort( $results, function ( $a, $b ) {
			return strnatcmp( $a->c_folder, $b->c_folder );
		} );

		$renderer = new Renderer();

		return $renderer->view( 'xml-statistics-custom-table', compact( 'results', 'atts' ) );
	}

	function shortcode_event_prints( $atts ) {
		$atts = shortcode_atts( array(
			'class'    => '',
			'the-code' => get_query_var( 'pg-code' ),
			'c_folder' => 'Prints',
			'c_size'   => 'Number of sequences'
		), $atts, 'photobooth-statistics-event-prints' );

		$atts = $this->handle_code( $atts );
		$time = $this->get_time( $atts );

		$where = [];

		if ( $time ) {
			$where[] = $time;
		}

		if ( $atts['the-code'] ) {

			if ( $where ) {
				$where[] = 'AND';
			}

			$where[] = sprintf( "c_output LIKE '%s'", '%' . esc_sql( $atts['the-code'] ) . '%' );
		}


		if ( $where ) {
			array_unshift( $where, 'WHERE' );
		}

		$table = XMLStatistics::table();

		$query = sprintf( 'SELECT COUNT(*) as c_size, c_prints as c_folder FROM %s %s GROUP BY c_prints', $table, implode( ' ', $where ) );

		if ( PG_LOG_VERBOSE ) {
			Helper::log( sprintf( '[ Statistic ] SQL Debug (shortcode_event_prints): %s ', $query ) );
		}

		$results = $this->execute_query( $query );

		if ( ! $results ) {
			return '';
		}

		$response = [];
		foreach ( $results as $result ) {
			$folder = explode( '\\', $result->c_folder );

			$folder = end( $folder );

			$result->c_folder = $folder;

			if ( array_key_exists( $folder, $response ) ) {
				$response[ $folder ]->c_size += $result->c_size;
				continue;
			}

			$response[ $folder ] = $result;
		}

		$results = array_values( $response );

		usort( $results, function ( $a, $b ) {
			return strnatcmp( $a->c_folder, $b->c_folder );
		} );

		$renderer = new Renderer();

		return $renderer->view( 'xml-statistics-custom-table', compact( 'results', 'atts' ) );
	}

	function shortcode_sharing_from( $atts ) {

		$atts = shortcode_atts( array(
			'date-format' => __( 'Y-m-d H:i:s', 'photobooth-gallery' ),
			'the-code'    => get_query_var( 'pg-code' )
		), $atts, 'photobooth-statistics-sharing-from' );

		$table = SocialMedia::table();

		return $this->shortcode_query( $atts, 'SELECT MIN(c_timestamp) AS d_from FROM %s %s', $table, 'c_file' );
	}

	function shortcode_sharing_to( $atts ) {

		$atts = shortcode_atts( array(
			'date-format' => __( 'Y-m-d H:i:s', 'photobooth-gallery' ),
			'the-code'    => get_query_var( 'pg-code' )
		), $atts, 'photobooth-statistics-sharing-to' );

		$table = SocialMedia::table();

		return $this->shortcode_query( $atts, 'SELECT MAX(c_timestamp) AS d_from FROM %s %s', $table, 'c_file' );
	}

	function shortcode_event_from( $atts ) {

		$atts = shortcode_atts( array(
			'date-format' => __( 'Y-m-d H:i:s', 'photobooth-gallery' ),
			'the-code'    => get_query_var( 'pg-code' )
		), $atts, 'photobooth-statistics-event-from' );

		$table = XMLStatistics::table();

		return $this->shortcode_query( $atts, 'SELECT MIN(c_timestamp) AS d_from FROM %s %s', $table, 'c_output' );
	}

	function shortcode_event_to( $atts ) {

		$atts = shortcode_atts( array(
			'date-format' => __( 'Y-m-d H:i:s', 'photobooth-gallery' ),
			'the-code'    => get_query_var( 'pg-code' )
		), $atts, 'photobooth-statistics-event-to' );

		$table = XMLStatistics::table();

		return $this->shortcode_query( $atts, 'SELECT MAX(c_timestamp) AS d_from FROM %s %s', $table, 'c_output' );
	}

	function shortcode_event_duration( $atts ) {
		$atts = shortcode_atts( array(
			'the-code' => get_query_var( 'pg-code' ),
			'format'   => true
		), $atts, 'photobooth-statistics-event-duration' );

		$table = XMLStatistics::table();

		$seconds = $this->shortcode_query( $atts, 'SELECT TIMESTAMPDIFF(SECOND, MIN(c_timestamp), MAX(c_timestamp) ) AS d_diff FROM %s %s', $table, 'c_output', false );

		if ( ! $seconds ) {
			return '';
		}

		$seconds = intval( $seconds->d_diff );

		if ( ! $atts['format'] ) {
			return $seconds;
		}

		$d1 = new \DateTime();
		$d2 = new \DateTime();
		$d2->add( new \DateInterval( 'PT' . $seconds . 'S' ) );

		$date_interval = $d2->diff( $d1 );

		$days    = $date_interval->format( '%a' );
		$hours   = $date_interval->format( '%h' );
		$minutes = $date_interval->format( '%i' );

		$text = [];

		if ( $days > 0 ) {
			$text[] = sprintf( '%d %s', $days, __( 'Days', 'photobooth-gallery' ) );
		}

		if ( $hours > 0 ) {

			if ( $text ) {
				$text[] = __( 'and', 'photobooth-gallery' );
			}

			$text[] = sprintf( '%d %s', $hours, __( 'Hours', 'photobooth-gallery' ) );
		}

		if ( $minutes > 0 ) {

			if ( $text ) {
				$text[] = __( 'and', 'photobooth-gallery' );
			}

			$text[] = sprintf( '%d %s', $minutes, __( 'Minutes', 'photobooth-gallery' ) );
		}


		return implode( ' ', $text );
	}


	private function get_time( $atts ) {

		$from = $atts['from'] ?? '';
		$to   = $atts['to'] ?? '';

		$from = strlen( $from ) ? $from : false;
		$to   = strlen( $to ) ? $to : false;

		if ( $from && $to ) {

			return sprintf( "c_timestamp BETWEEN '%s' AND '%s'", esc_sql( $from ), esc_sql( $to ) );
		}

		if ( $from ) {

			return sprintf( "c_timestamp >= '%s'", esc_sql( $from ) );
		}

		if ( $to ) {

			return sprintf( "c_timestamp <= '%s'", esc_sql( $to ) );
		}

		return '';
	}

	private function execute_query( $query ) {
		global $wpdb;

		$results = $wpdb->get_results( $query );

		if ( ! is_array( $results ) ) {
			return [];
		}

		return $results;
	}

	private function get_results( $select, $where = [] ) {

		$table = SocialMedia::table();

		if ( ! $select ) {
			return 0;
		}

		if ( sizeof( $select ) === 1 ) {
			$select = reset( $select );
		} else {
			$select = implode( ', ', $select );
		}

		if ( $where ) {
			array_unshift( $where, 'WHERE' );
		}

		$query = sprintf( 'SELECT %s FROM %s %s', $select, $table, implode( ' ', $where ) );

		if ( PG_LOG_VERBOSE ) {
			Helper::log( sprintf( '[ Statistic ] SQL Debug: %s ', $query ) );
		}

		$result = $this->execute_query( $query );

		if ( ! is_array( $result ) ) {
			return 0;
		}

		return $result;
	}

	public function top_image( $type ) {

		$atts = $this->prepare_parameters();
		$time = $this->get_time( $atts );

		$where = [
			sprintf( "c_type = '%s'", esc_sql( $type ) )
		];

		if ( $time ) {
			$where[] = 'AND';
			$where[] = $time;
		}

		if ( $atts['source'] ) {
			$where[] = 'AND';
			$where[] = sprintf( "c_config = '%s'", esc_sql( $atts['source'] ) );
		}

		if ( strlen( $this->code ) ) {
			$where[] = 'AND';
			$where[] = sprintf( "c_file LIKE '%s'", '%' . esc_sql( $this->code ) . '%' );
		}

		$where[] = 'GROUP BY c_reference';
		$where[] = 'ORDER BY r_size DESC';

		$result = $this->get_results( [ 'COUNT(*) AS r_size', 'c_reference AS attachment_id' ], $where );

		$result = reset( $result );

		if ( ! $result ) {
			return 0;
		}

		return [ $result->r_size, $result->attachment_id ];

	}

	private function handle_xml_counts( $network = false, $type = SocialMedia::SHARE, $atts = [] ) {

		$stats = new XMLStatistics();

		$where = '';

		if ( $this->code ) {
			$where = sprintf( "c_output LIKE '%s'", '%' . esc_sql( $this->code ) . '%' );
		}

		$result = $stats->get_statistics( array_merge( $atts, [
			'where' => $where,
			'group' => false
		] ) );

		if ( ! $result ) {
			return 0;
		}

		// We removed the group by, thus we will always get one result
		$result = reset( $result );

		// network = xml- .... (contains the field we want to select)
		$field = substr( $network, 4 );

		$value = intval( $result->{$field} );

		if ( $type !== 'average' ) {
			return $value;
		}

		$atts = [
			'format' => false
		];

		$calc_method = 'value_every_x_seconds';

		// Average photos per SESSION not Time Interval
		if ( $network === 'xml-photos' ) {

			$seconds     = $this->count_by_network( 'xml-sessions', SocialMedia::VIEW );
			$calc_method = 'per_session';

		} else {
			$seconds = $this->shortcode_event_duration( $atts );
		}

		if ( ! $seconds ) {
			$seconds = 1;
		}

		if ( $value === 0 ) {
		    $value = 1;
        }

		if ( $calc_method === 'value_every_x_seconds' ) {
			return round( $seconds / $value );
		}

		if ( $calc_method === 'per_session' ) {
			return round( $value / $seconds, 1 );
		}

		return $value;
	}

	private function handle_wordpress_media_counts( $network, $atts ) {

		global $wpdb;

		$where = [
			"post_meta.meta_value = 'photobooth-gallery-image'",
			'AND',
			"posts.post_type = 'attachment'"
		];

		if ( strpos( $network, 'image' ) !== false ) {
			$where[] = 'AND';
			$where[] = "posts.post_mime_type LIKE 'image%'";
		}

		if ( strpos( $network, 'video' ) !== false ) {
			$where[] = 'AND';
			$where[] = "posts.post_mime_type LIKE 'video%'";
		}
		if ( strlen( $this->code ) ) {
			$where[] = 'AND';
			$where[] = sprintf( "post_meta_2.meta_value LIKE '%s'", '%' . esc_sql( $this->code ) . '%' );
		}

		$query = sprintf( "SELECT COUNT(posts.ID) AS r_size 
								 FROM {$wpdb->posts} as posts 
								 	LEFT JOIN {$wpdb->postmeta} as post_meta ON (post_meta.post_id = posts.id AND post_meta.meta_key = 'type')
								  	LEFT JOIN {$wpdb->postmeta} as post_meta_2 ON (post_meta_2.post_id = posts.id AND post_meta_2.meta_key = 'real_filename') 
								 WHERE %s", implode( ' ', $where ) );

		if ( PG_LOG_VERBOSE ) {
			Helper::log( sprintf( '[ Statistic ] SQL Debug: %s ', $query ) );
		}

		$results = $wpdb->get_results( $query );

		if ( ! $results ) {
			return 0;
		}

		if ( ! is_array( $results ) ) {
			return 0;
		}

		$result = reset( $results );

		return $result->r_size;
	}

	public function reset() {
	    $table = SocialMedia::table();

	    global $wpdb;

        return $wpdb->query( sprintf('DELETE FROM %s', $table ) );
    }

	public function group_by_agent( $filter, $type = SocialMedia::SHARE ) {
		$atts = $this->prepare_parameters();
		$time = $this->get_time( $atts );

		$where = [
			sprintf( "c_type = '%s'", esc_sql( $type ) )
		];

		if ( $time ) {
			$where[] = 'AND';
			$where[] = $time;
		}

		if ( $atts['source'] ) {
			$where[] = 'AND';
			$where[] = sprintf( "c_config = '%s'", esc_sql( $atts['source'] ) );
		}

		if ( strlen( $this->code ) ) {
			$where[] = 'AND';
			$where[] = sprintf( "c_file LIKE '%s'", '%' . esc_sql( $this->code ) . '%' );
		}

		$result = $this->get_results( [ 'c_user_agent' ], $where );

		if ( ! $result ) {
			return false;
		}

		$results = [
			'total'   => sizeof( $result ),
			'unknown' => 0,
			'data'    => [],
		];

		foreach ( $result as $agent ) {
			$agent = $agent->c_user_agent;

			if ( ! $agent ) {

				$results['unknown'] ++;

				continue;
			}

			if ( ! strlen( $agent ) ) {

				$results['unknown'] ++;

				continue;
			}

			$parser = new Parser( $agent );

			if ( $filter === 'devices' ) {

				$data = $results['data'];

				$device = $this->cleanup_device_agent( $parser->device, $agent );

				if ( $device === false ) {
					continue;
				}

				if ( ! array_key_exists( $device, $data ) ) {
					$data[ $device ] = 0;
				}

				$data[ $device ] ++;

				$results['data'] = $data;

				continue;

			}

			if ( $filter === 'browser' ) {

				$data = $results['data'];

				$browser = $parser->browser;

				if ( ! array_key_exists( $browser->getName(), $data ) ) {
					$data[ $browser->getName() ] = 0;
				}

				$data[ $browser->getName() ] ++;

				$results['data'] = $data;

				continue;

			}

			if ( $filter === 'os' ) {

				$data = $results['data'];

				$os = $parser->os;

				$name = $this->cleanup_user_agent( $os, $agent );

				if ( ! array_key_exists( $name, $data ) ) {
					$data[ $name ] = 0;
				}

				$data[ $name ] ++;

				$results['data'] = $data;

				continue;

			}

		}

		return $results;
	}

	private function cleanup_device_agent( Device $device, $agent ) {

		if ( $device->type === DeviceType::BOT ) {
			return false;
		}

		if ( $device->type === DeviceType::TABLET ) {
			return DeviceType::TABLET;
		}

		if ( $device->type === DeviceType::MOBILE ) {
			return DeviceType::MOBILE;
		}

		if ( $device->type === DeviceType::DESKTOP ) {
			return DeviceType::DESKTOP;
		}

		return __( 'Unknown', 'photobooth-gallery' );
	}

	private function cleanup_user_agent( Os $os, $agent ) {

		if ( $os->getName() ) {
			return $os->getName();
		}

		$agent = strtolower( $agent );

		if ( strpos( $agent, 'whatsapp' ) !== false ) {
			return 'Whatsapp';
		}

		if ( strpos( $agent, 'facebookexternalhit' ) !== false ) {
			return 'Facebook Share';
		}

		return __( 'Unknown', 'photobooth-gallery' );
	}

	/**
	 * COUNTS shares or views in a selected time frame for a given network or all available networks
	 *
	 * @param bool $network
	 * @param string $type
	 * @param array $query_params
	 *
	 * @return int
	 */
	public function count_by_network( $network = false, $type = SocialMedia::SHARE, $query_params = [] ) {

		$atts = $this->prepare_parameters();
		$time = $this->get_time( $atts );

		// We want to receive xml data
		if ( strpos( $network, 'xml-' ) === 0 ) {
			return $this->handle_xml_counts( $network, $type, $atts );
		}

		// We want to receive local Wordpress data
		if ( strpos( $network, 'wp-' ) === 0 ) {
			return $this->handle_wordpress_media_counts( $network, $atts );
		}

		$group_by = false;
		$order_by = false;

		// Special Case: _all is the keyword for no specific network selected
		if ( $network === '_all' ) {
			$network = false;
		}

		// Special Case: Unique requests are grouped by the requested file
		if ( $type === 'unique' ) {
			$type     = SocialMedia::VIEW;
			$group_by = 'GROUP BY c_file';
		}

		// Download videos
		if ( $network === 'download-video' ) {
			$query_params[] = 'AND';
			$query_params[] = "c_mime_type LIKE 'video%'";
		}

		// Download images
		if ( $network === 'download-image' ) {
			$query_params[] = 'AND';
			$query_params[] = "c_mime_type LIKE 'image%'";
		}

		// Special Case: Download is not view trackable
		if ( strpos( $network, 'download' ) !== false ) {
			$type    = SocialMedia::SHARE;
			$network = 'download';
		}

		// Special Case: Network comes from the Log File Actions, Type is always view
		if ( strpos( $network, '-clicked' ) !== false ) {
			$type = SocialMedia::VIEW;
		}

		// Special Case: Network is all viewed, which combines all logfile trackings
		if ( $network === 'all-viewed' ) {
			$network = false;
			$type    = SocialMedia::VIEW;

			$query_params[] = 'AND';
			$query_params[] = "c_network LIKE '%-clicked'";
		}

		$where = [
			sprintf( "c_type = '%s'", esc_sql( $type ) )
		];

		if ( $time ) {
			$where[] = 'AND';
			$where[] = $time;
		}

		if ( $atts['source'] ) {
			$where[] = 'AND';
			$where[] = sprintf( "c_config = '%s'", esc_sql( $atts['source'] ) );
		}

		if ( $network !== false ) {
			$where[] = 'AND';
			$where[] = sprintf( "c_network = '%s'", esc_sql( $network ) );
		}

		if ( strlen( $this->code ) ) {
			$where[] = 'AND';
			$where[] = sprintf( "c_file LIKE '%s'", '%' . esc_sql( $this->code ) . '%' );
		}


		// Add potential other query parameters to query
		array_push( $where, implode( ' ', $query_params ) );

		// Add group by
		if ( $group_by ) {
			$where[] = $group_by;
		}

		// Add order by
		if ( $order_by ) {
			$where[] = $order_by;
		}

		$result = $this->get_results( [ 'COUNT(*) as r_size' ], $where );

		// GROUP BYs count the result size instead a single count
		if ( $group_by ) {
			return sizeof( $result );
		}

		$result = reset( $result );

		if ( ! $result ) {
			return 0;
		}

		return intval( $result->r_size );
	}

	private function prepare_parameters() {

		$atts = [
			'the-code' => get_query_var( 'pg-code' ),
			'source'   => false
		];

		if ( $this->code ) {
			$atts['the-code'] = $this->code;
		}

		$atts = $this->handle_code( $atts );

		return $atts;
	}

}
