<?php

// If this file is called directly, abort.
if (!defined('WPINC')) {
    die;
}

use Codevision\Compatibility\SimpleCheck;
use Codevision\Environment;
use Codevision\Licensing\Manager;
use Photobooth\Migration;
use Twig\Environment as EnvironmentAlias;

define('PG_LOG_VERBOSE', false);
define('PG_DEBUG', false);
define('PG_COOKIE', true);

define('PHOTOBOOTH_GALLERY_VERSION', '3.5.6');

define('PG_NAMESPACE', 'pg-');
define('PG_SLUG', 'photobooth-gallery');

define('PG_BASE_DIR', __DIR__);
define('PG_BOOTSTRAP_FILE', __FILE__);
define('PG_IMAGE_URL_PUBLIC', plugin_dir_url(__FILE__).'public/img/');
define('PG_IMAGE_URL_ADMIN', plugin_dir_url(__FILE__).'admin/img/');
define('PG_IMAGE_PATH_ADMIN', PG_BASE_DIR.'/admin/img/');
define('PG_CACHE_PATH', PG_BASE_DIR.'/cache');
define('PG_CACHE_URL', plugin_dir_url(__FILE__).'cache');
define('PG_TEMPLATES_DIR', PG_BASE_DIR.'/templates');

define('PG_MIN_PHP', '8.2');
define('PG_MIN_WP', '6.4.2');
define('PG_DEV_MAIL', 'photobooth-gallery-bugs@codevision.io');

define('PG_MIN_ELEMENTOR', '3.16.1');


if (is_dir(__DIR__ . '/vendor') === false) {
    $success = include_once __DIR__ . '/preload.php';
    if ($success === false) {
        return;
    }
}

include_once __DIR__.'/vendor/autoload.php';

/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              https://www.photobooth-deluxe.de/
 * @since             3.0.0
 * @package           Photobooth_Gallery
 *
 * @wordpress-plugin
 * Plugin Name:       [ Photobooth Deluxe ] Photobooth Gallery
 * Plugin URI:        https://www.photobooth-deluxe.de/
 * Description:       View, share and configure photobooth gallery images in wordpress
 * Version:           3.5.6
 * Requires at least: 6.2.0
 * Tested up to:      6.6.1
 * Requires PHP:      8.2
 * Author:            photobooth-deluxe
 * Author URI:        https://www.photobooth-deluxe.de/
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       photobooth-gallery
 * Domain Path:       /languages
 */

// These are here for the php-pot-file generator to find plugin translations
$translations = [
    __('Photobooth Gallery', 'photobooth-gallery'),
    __('[ Photobooth Deluxe ] Photobooth Gallery', 'photobooth-gallery'),
    __('https://www.photobooth-deluxe.de/', 'photobooth-gallery'),
    __('View, share and configure photobooth gallery images in wordpress', 'photobooth-gallery'),
];

function activate_photobooth_gallery()
{

    Photobooth_Gallery_Activator::activate();
}

function deactivate_photobooth_gallery()
{

    Photobooth_Gallery_Deactivator::deactivate();
}

register_activation_hook(__FILE__, 'activate_photobooth_gallery');

function pg_get_license_manager()
{

    $link = '"'.admin_url('admin.php?page=acf-options-global-settings').'"';

    $env = new Environment();

    $env->setPluginName(__('Photobooth Gallery', 'photobooth-gallery'))
        ->setBaseFile(PG_BOOTSTRAP_FILE)
        ->setVersion(PHOTOBOOTH_GALLERY_VERSION)
        ->setBasePath(PG_BASE_DIR)
        ->setSlug(PG_SLUG)
        ->setBaseUrl(plugin_dir_url(PG_BOOTSTRAP_FILE))
        ->setVendorUrl(plugin_dir_url(PG_BOOTSTRAP_FILE).'vendor');

    $log_folder = __DIR__.'/log';
    $cache_folder = __DIR__.'/cache';
    $booth_folder = WP_CONTENT_DIR.'/../../photobooth';
    $licenseServerUrl = 'https://license.codevision.io';

    $manager = Manager::getInstanceWithCompatiblityCheck($env);

    $manager->setApiServer($licenseServerUrl)
        ->setAuthKey('CA26F767-A89A-4DFC-8438-B3A31B475DAE')
        ->setNamespace(PG_NAMESPACE)
        ->setLicenseInfoField('field_5b7e83c843f11')
        ->setLicenseKeyName('pg_api_key')
        ->setLicenseViewSlug('global-settings')
        ->setAdditionalData([
            'purchase_id' => get_field('pg_purchase_id', 'option', false),
            'plugin' => PG_SLUG,
            'version' => PHOTOBOOTH_GALLERY_VERSION,
        ])
        ->setMessages([
            'headline' => 'Photobooth-Gallery Plugin',
            'missing_license_admin_notice' => [
                'Could not verify the License Key. Please enter a new License Key <a href=%s>here</a>. As long as no valid License Key is found, the Plugin will not work correctly.',
                $link,
            ],
            'trial_period_admin_notice' => 'You are using Photobooth Gallery without a valid License Key. You have <strong>%d</strong> days left for your trial.',
            'trial' => 'You are using Photobooth Gallery without a valid License Key. You have <strong>%d</strong> days left for your trial.',
            'trial_one_day_left' => 'You are using Photobooth Gallery without a valid License Key. You have only <strong>one</strong> day left for your trial.',
        ])
        ->getCompatibilityCheck()
        ->setSuccessFunction(function () use ($manager) {

            register_deactivation_hook(__FILE__, 'deactivate_photobooth_gallery');

            new Migration(get_option(PG_NAMESPACE.'installed-version', false));

            $plugin = new Photobooth_Gallery($manager);
            $plugin->run();
        })
        ->setTitle('Photobooth Gallery Compatibility Check')
        ->setEscalationScore(50)
        ->add(
            new SimpleCheck(
                sprintf('License server secure connection test [ %s ] ', $licenseServerUrl),
                sprintf(
                    'We cannot make a secure connection to our license server @ %s! Please check your web-firewall, security plugin(s) and hosting settings!',
                    $licenseServerUrl
                ),
                function () use ($licenseServerUrl) {

                    $response = wp_remote_get($licenseServerUrl);

                    return !($response instanceof \WP_Error);
                },
                50
            )
        )
        ->add(
            new SimpleCheck(
                sprintf('PHP Version Check [ Required at least %s] ', PG_MIN_PHP),
                sprintf(
                    'Your PHP Version %s is to low, please upgrade to at least Version %s to use this Plugin!',
                    PHP_VERSION,
                    PG_MIN_PHP
                ),
                function () {

                    return version_compare(PHP_VERSION, PG_MIN_PHP, '>=');
                },
                50
            )
        )
        ->add(
            new SimpleCheck(
                sprintf('Wordpress Version Check [ Required at least %s ]', PG_MIN_WP),
                sprintf(
                    'Your Wordpress version is too old for this plugin. At least, Wordpress version %s is required!',
                    PG_MIN_WP
                ),
                function () {

                    global $wp_version;

                    return version_compare($wp_version, PG_MIN_WP, '>=');
                },
                50
            )
        )
        ->add(
            new SimpleCheck(
                sprintf('Elementor Version Check (if installed) [ Required at least %s ]', PG_MIN_ELEMENTOR),
                sprintf(
                    'Your installed Elementor version is too old for this plugin. At least, Elementor version %s is required!',
                    PG_MIN_ELEMENTOR
                ),
                function () {

                    return version_compare(ELEMENTOR_VERSION, PG_MIN_ELEMENTOR, '>=');
                },
                50
            )
        )
        ->add(
            new SimpleCheck(
                sprintf('PHP Timeout Limit [ %s recommended ]', '>60 seconds'),
                sprintf('Please increase your PHP Timeout Limit to at least %s', '60 seconds'),

                function () {

                    $current_limit = intval(ini_get('max_execution_time'));

                    if ($current_limit === 0) {
                        return true;
                    }

                    return $current_limit >= 60;

                },
                10
            )
        )
        ->add(
            new SimpleCheck(
                sprintf('PHP Timeout Change Check'),
                sprintf('Please increase your PHP Timeout Limit to at least %s', '60 seconds'),

                function () {

                    $current_limit = intval(ini_get('max_execution_time'));

                    if ($current_limit === 0) {
                        return true;
                    }

                    @set_time_limit(300);

                    $limit = ini_get('max_execution_time');

                    if ($limit !== '300') {

                        return false;
                    }

                    @set_time_limit($current_limit);

                    return true;

                },
                10
            )
        )
        ->add(
            new SimpleCheck(
                sprintf('PHP Memory Limit Check [ %s recommended ]', '>=256M'),
                sprintf('Please increase your PHP Memory Limit to at least %s', '256M'),

                function () {

                    $required = 256 * 1024 * 1024;

                    $limit = ini_get('memory_limit');

                    sscanf($limit, '%u%c', $number, $suffix);

                    if (!isset($suffix)) {
                        return false;
                    }

                    if (strtolower($suffix) === 'g') {
                        return true;
                    }

                    $number *= pow(1024, strpos(' KMG', strtoupper($suffix)));

                    return $number >= $required;

                },
                10
            )
        )
        ->add(
            new SimpleCheck(
                sprintf('Testing Writing permissions to log folder [ %s ]', $log_folder),
                sprintf('Could not write to log folder %s, please set the permissions to 777', $log_folder),
                function () use ($log_folder) {

                    if (!is_dir($log_folder)) {
                        @mkdir($log_folder, 0755, true);
                    }

                    $test_file = $log_folder.'/.test';

                    if (!@touch($test_file)) {
                        return false;
                    }

                    if (!@unlink($test_file)) {
                        return false;
                    }

                    return true;
                },
                50
            )
        )
        ->add(
            new SimpleCheck(
                sprintf('Testing Writing permissions to cache folder [ %s ]', $cache_folder),
                sprintf('Could not write to cache folder %s, please set the permissions to 777', $cache_folder),
                function () use ($cache_folder) {

                    if (!is_dir($cache_folder)) {
                        @mkdir($cache_folder, 0755, true);
                    }

                    $test_file = $cache_folder.'/.test';

                    if (!@touch($test_file)) {
                        return false;
                    }

                    if (!@unlink($test_file)) {
                        return false;
                    }

                    return true;
                },
                50
            )
        )
        ->add(
            new SimpleCheck(
                sprintf('Testing creating photobooth source directory [ %s ]', $booth_folder),
                sprintf(
                    'Could not create photobooth source directory %s, please create it by yourself or choose a different folder in the configuration',
                    $booth_folder
                ),
                function () use ($booth_folder) {

                    if (is_dir($booth_folder)) {
                        return true;
                    }

                    if (!@mkdir($booth_folder, 0755)) {
                        return false;
                    }

                    if (!@rmdir($booth_folder)) {
                        return false;
                    }

                    return true;
                },
                10
            )
        )
        ->add(
            new SimpleCheck(
                sprintf('Testing potential available Twig Extension [ Required at least 2.7.2 ]'),
                sprintf(
                    'You are using at least one Plugin which adds Twig Version 2.6 or lower, please upgrade those plugin(s) or deactivate them, otherwise you could get PHP error messages while using this plugin.'
                ),
                function () {

                    if (!class_exists('Twig_Environment')) {
                        return true;
                    }

                    return version_compare(EnvironmentAlias::VERSION, '2.7.2', '>=');

                },
                30
            )
        )
        ->add(
            new SimpleCheck('PHP Extensions FILEINFO', 'Missing required PHP Extension FILEINFO', function () {

                return extension_loaded('fileinfo');
            }, 60)
        )
        ->add(
            new SimpleCheck('PHP Extensions GD', 'Missing required PHP Extension GD', function () {

                return extension_loaded('gd');
            }, 60)
        )
        ->add(
            new SimpleCheck('PHP Extensions ICONV', 'Missing required PHP Extension ICONV', function () {

                return extension_loaded('iconv');
            }, 60)
        )
        ->add(
            new SimpleCheck('PHP Extensions ZIP', 'Missing PHP Extension ZIP', function () {

                return extension_loaded('zip');
            }, 10)
        )
        ->add(
            new SimpleCheck('PHP Extensions IMAGEICK', 'Missing PHP Extension imagick', function () {

                return extension_loaded('imagick');
            }, 10)
        )
        ->add(
            new SimpleCheck('PHP Extensions XML', 'Missing PHP Extension XML', function () {

                return extension_loaded('xml');
            }, 10)
        )
        ->add(
            new SimpleCheck('PHP Extensions Simple XML', 'Missing PHP Extension Simple XML', function () {

                return extension_loaded('simplexml');
            }, 10)
        );

    return $manager;
}

add_action('pbg/acfpolylang/register-textdomain', function(){
    Photobooth_Gallery::registerTextDomains();
});

add_action('acf/init', function () {
    $license_manager = pg_get_license_manager();
    $license_manager->init();
});

add_action('plugins_loaded', function () {
    Photobooth_Gallery_Admin::pluginInit();
});

new Photobooth_Gallery_i18n();
