(function ($) {

    'use strict';
    var $document = $(document),
        on = function (event, selector, fn) {
            $document.on(event, selector, fn);
        };

    $.urlParam = $.urlParam || function (name, str) {

        if (!str) {
            str = window.location.search;
        }

        var results = new RegExp('[\?&]' + name + '=([^&#]*)').exec(str);

        return (results !== null) ? results[1] || 0 : false;
    };

    const pg_data = window.pg_data || {};
    const $body = $(document.body);

    const requestNewImages = function (timeOut, dataCallback, successCallback, failCallback) {

        if ($document.data('ajax.requestNewImages.in_progess') === true) {
            return;
        }

        $document.data('ajax.requestNewImages.in_progess', true);

        $.ajax({
            type: 'POST',
            cache: false,
            url: pg_data.ajaxurl,
            data: {
                'form': dataCallback(),
                'action': 'pg-next-page',
                'csrf': pg_data['pg-next-page']
            }
        })
            .done((response) => {

                if (successCallback(response) === false) {
                    return;
                }

                if (timeOut === 0) {
                    return;
                }

                window.setTimeout(requestNewImages, timeOut, timeOut, dataCallback, successCallback, failCallback);
            })
            .fail(failCallback)
            .complete(() => {
                $document.data('ajax.requestNewImages.in_progess', false);
            })
    }

    const dataCallback = function () {
        const $gallery = $('div.the-gallery.photobooth-gallery-swiper-tv');

        return $.extend({
            'loaded': $gallery.find('div.swiper-slide:not(.swiper-slide-duplicate)').length,
            'refresh': true,
        }, $gallery.data());
    };

    const successCallback = function (response) {
        const gallery = document.querySelector('.swiper-tv').swiper;

        if (!gallery) {
            return false;
        }

        if (response.error === true) {
            return false;
        }

        if (response.hasElements === false) {
            return true;
        }

        const $content = $(response.data),
            $contentGallery = $content.find('div.the-gallery'),
            $gallery = $('div.photobooth-gallery-swiper-tv');

        let $slides = $content.find('div.swiper-slide');

        $slides = $slides.filter((index, item) => {
            let $item = $(item);

            return $('#' + $item.attr('id')).length === 0;
        });

        if ($slides.length === 0) {
            return true;
        }

        $gallery.data('page', $contentGallery.data('page'));
        $gallery.data('total', $contentGallery.data('total'));

        gallery.autoplay.stop();
        gallery.appendSlide($slides);

        gallery.update();
        gallery.autoplay.start();

        return true;
    };

    const failCallback = function (response) {

    }


    $(function () {
        let refreshTimeout = $.urlParam('refresh') || 0,
            $swiper = $('.swiper-tv');

        $swiper.on('init.swiper', () => {
            const swiper = document.querySelector('.swiper-tv').swiper;

            swiper.on('slideChange', (swiper) => {
                const $gallery = $('div.photobooth-gallery-swiper-tv');

                let dataSet = dataCallback(),
                    total = parseInt($gallery.data('total')),
                    currentSlide = swiper.realIndex + 1,
                    reloadAt = Math.max(1, dataSet.loaded - 3);

                if (total === dataSet.loaded) {
                    return;
                }

                if (currentSlide < reloadAt) {
                    return;
                }

                requestNewImages(0, () => {
                    return $.extend(dataSet, {'refresh': false});
                }, successCallback, failCallback);
            });

            if (refreshTimeout === 0) {
                return;
            }

            if (refreshTimeout <= 3000) {
                refreshTimeout *= 1000;
            }

            refreshTimeout = Math.max(10000, refreshTimeout);

            window.setTimeout(
                requestNewImages,
                refreshTimeout,
                refreshTimeout,
                dataCallback,
                successCallback,
                failCallback
            );
        });
    });

})(jQuery);

//# sourceMappingURL=maps/smart-tv.min.js.map
