(function ($) {

    'use strict';
    var $document = $(document),
        on = function (event, selector, fn) {
            $document.on(event, selector, fn);
        };

    window.cvJsRequire = window.cvJsRequire || {
        'require': function () {
            console.error('cvJsRequire is required for this to work!')
        }
    }

    $(function () {
        const slideChange = function () {

            const swiper = this,
                $currentSlide = $(this.slides[this.activeIndex]),
                $swiper = $(swiper.el);

            swiper.enable();

            if ($currentSlide.find('video').length === 0) {
                return;
            }

            let $video = $currentSlide.find('video');

            if ($video.hasClass('video-js-initialized') === true) {
                const player = videojs.getPlayer($video[0]);
                player.currentTime(0);
                player.play();
                return;
            }

            if ($video.hasClass('video-js-attached') === true) {
                return;
            }

            $video.addClass('video-js-attached');

            cvJsRequire.require(['video.js', 'video-js.css'], function () {

                $currentSlide.find('.swiper-lazy-preloader').remove();

                const player = videojs($video[0], {
                    controls: $swiper.data('autoplay') === false,
                    autoplay: 'any',
                    preload: 'metadata',
                    fluid: true
                });

                player.on('ended', function () {
                    player.currentTime(0);
                    swiper.enable();
                    if ($swiper.data('autoplay') === true) {
                        swiper.autoplay.start();
                    }
                });

                player.on('play', function () {
                    swiper.disable();
                    swiper.autoplay.stop();
                });

                player.on('pause', function () {
                    swiper.enable();
                });

                player.on('ready', function () {
                    $video.addClass('video-js-initialized');
                    swiper.autoplay.stop();
                    _.delay(() => {
                        swiper.update();
                        player.muted(true);
                        player.play();
                    }, 100);
                });

            });
        };

        const requestNewImages = function (dataCallback, successCallback, failCallback) {

            if ($document.data('ajax.requestNewImages.in_progess') === true) {
                return;
            }

            $document.data('ajax.requestNewImages.in_progess', true);

            $.ajax({
                type: 'POST',
                cache: false,
                url: pg_data.ajaxurl,
                data: {
                    'form': dataCallback(),
                    'action': 'pg-next-page',
                    'csrf': pg_data['pg-next-page']
                }
            })
                .done(successCallback)
                .fail(failCallback)
                .complete(() => {
                    $document.data('ajax.requestNewImages.in_progess', false);
                })
        };

        const dataCallback = function () {
            const $gallery = $('div.the-gallery.photobooth-gallery-swiper');

            return $.extend({
                'loaded': $gallery.find('.main-swiper .swiper-slide:not(.swiper-slide-duplicate)').length,
                'refresh': false,
            }, $gallery.data());
        };

        cvJsRequire.require(['swiper-bundle.js', 'swiper-bundle.css'], function () {
            const reloadImages = (currentSwiper) => {
                const $gallery = $('div.photobooth-gallery-swiper');

                let dataSet = dataCallback(),
                    total = parseInt($gallery.data('total')),
                    currentSlide = currentSwiper.realIndex + 1,
                    reloadAt = Math.max(1,dataSet.loaded - 3);

                if (total === dataSet.loaded) {
                    return;
                }

                if (currentSlide < reloadAt) {
                    return;
                }

                requestNewImages(dataCallback, (response) => {
                    if (response.error === true) {
                        return false;
                    }

                    if (response.hasElements === false) {
                        return true;
                    }

                    const $content = $(response.data),
                        $contentGallery = $content.find('div.the-gallery'),
                        $gallery = $('div.photobooth-gallery-swiper'),
                        filterFn = (index, item) => {
                            let $item = $(item);

                            return $('#' + $item.attr('id')).length === 0;
                        };

                    let $slides = $contentGallery.find('.swiper.main-swiper .swiper-slide'),
                        $thumbSlides = $contentGallery.find('.swiper.thumb-swiper .swiper-slide');

                    $slides = $slides.filter(filterFn);

                    if ($slides.length === 0) {
                        return true;
                    }

                    $thumbSlides = $thumbSlides.filter(filterFn);

                    $gallery.data('page', $contentGallery.data('page'));
                    $gallery.data('total', $contentGallery.data('total'));

                    thumbSwiper.appendSlide($thumbSlides);
                    swiper.appendSlide($slides);

                    thumbSwiper.update();
                    swiper.update();

                    let $images = $(swiper.slides).find('img.zoomable-image');

                    $images.on('load', function (){
                        swiper.update();
                    });

                    const zoom = $document.data('pbg-zoom') || false;

                    if (zoom) {
                        zoom.attach($images.map((idx, item) => item).toArray());
                    }

                    return true;
                }, () => {
                });
            };

            const thumbSwiper = new Swiper(".thumb-swiper", {
                loop: true,
                spaceBetween: 10,
                freeMode: true,
                watchSlidesProgress: true,
                mousewheelControl: true,
                mousewheel: {
                    releaseOnEdges: true,
                },
                slidesPerView: 4,
                navigation: {
                    nextEl: '.swiper-button-next',
                    prevEl: '.swiper-button-prev',
                },
                pagination: false,
                autoHeight: true,
                on : {
                    slideChange : reloadImages
                }
            });

            const swiper = new Swiper(".main-swiper", {
                loop: true,
                spaceBetween: 10,
                navigation: false,
                pagination: false,
                preloadImages: false,
                lazy: true,
                autoHeight: true,
                thumbs: {
                    swiper: thumbSwiper,
                },
                on: {
                    init: slideChange,
                    slideChangeTransitionEnd : slideChange,
                    slideChange : reloadImages
                },
            });
        });

        on('show.bs.modal', (ev) => {
            let $modal = $(ev.target),
                $socialContainer = $(ev.relatedTarget).closest('.social-media-container');

            if ($socialContainer.length === 0) {
                return;
            }

            const swiper = $socialContainer.closest('.swiper')[0].swiper;

            $modal
                .data('smc-id', $socialContainer.attr('id'))
                .find('.modal-body').html($socialContainer.find('.copy-content').html());
            $socialContainer.addClass('pg-visible');
            swiper.disable();
        });

        on('hide.bs.modal', (ev) => {
            let $modal = $(ev.target),
                $socialContainer = $('#' + $modal.data('smc-id'));

            if ($socialContainer.length === 0) {
                return;
            }

            const swiper = $socialContainer.closest('.swiper')[0].swiper;

            $socialContainer.removeClass('pg-visible');
            swiper.enable();
        });

        on('pg.admin-remove-images', (ev, data) => {
            location.reload();
        });

        if ($('.zoomable-image').length > 0) {
            cvJsRequire.require('zoom.min.js', () => {
                $document.data('pbg-zoom', Zoom(".zoomable-image", {
                    background: "auto"
                }));
            });
        }
    });

})(jQuery);

//# sourceMappingURL=maps/swiper-thumb.min.js.map
