(function ($) {

    'use strict';
    var $document = $(document),
        on = function (event, selector, fn) {
            $document.on(event, selector, fn);
        };

    window.cvJsRequire = window.cvJsRequire || {
        'require': function () {
            console.error('cvJsRequire is required for this to work!')
        }
    }

    $.urlParam = $.urlParam || function (name, str) {

        if (!str) {
            str = window.location.search;
        }

        var results = new RegExp('[\?&]' + name + '=([^&#]*)').exec(str);

        return (results !== null) ? results[1] || 0 : false;
    };

    const effects = [
        'coverflow',
        'cards',
        'flip',
        'cube',
        'fade'
    ];

    $(function () {
        const slideChange = function () {

            const swiper = this,
                $currentSlide = $(this.slides[this.activeIndex]),
                $swiper = $(swiper.el);
            swiper.enable();

            if ($currentSlide.find('video').length === 0) {
                return;
            }

            let $video = $currentSlide.find('video');

            if ($video.hasClass('video-js-initialized') === true) {
                const player = videojs.getPlayer($video[0]);
                player.currentTime(0);
                player.play();
                return;
            }

            if ($video.hasClass('video-js-attached') === true) {
                return;
            }

            $video.addClass('video-js-attached');

            cvJsRequire.require(['video.js', 'video-js.css'], function () {

                $currentSlide.find('.swiper-lazy-preloader').remove();

                const player = videojs($video[0], {
                    controls: $swiper.data('autoplay') === false,
                    autoplay: 'any',
                    preload: 'metadata',
                    fluid: true
                });

                player.on('ended', function () {
                    player.currentTime(0);
                    swiper.enable();
                    swiper.autoplay.start();
                });

                player.on('play', function () {
                    swiper.disable();
                    swiper.autoplay.stop();
                });

                player.on('pause', function () {
                    swiper.enable();
                });

                player.on('ready', function () {
                    $video.addClass('video-js-initialized');
                    swiper.autoplay.stop();
                    _.delay(() => {
                        swiper.update();
                        player.muted(true);
                        player.play();
                    }, 100);
                });

            });
        };

        const requestNewImages = function (dataCallback, successCallback, failCallback) {

            if ($document.data('ajax.requestNewImages.in_progess') === true) {
                return;
            }

            $document.data('ajax.requestNewImages.in_progess', true);

            $.ajax({
                type: 'POST',
                cache: false,
                url: pg_data.ajaxurl,
                data: {
                    'form': dataCallback(),
                    'action': 'pg-next-page',
                    'csrf': pg_data['pg-next-page']
                }
            })
                .done(successCallback)
                .fail(failCallback)
                .complete(() => {
                    $document.data('ajax.requestNewImages.in_progess', false);
                })
        };

        const dataCallback = function () {
            const $gallery = $('div.the-gallery.photobooth-gallery-swiper');

            return $.extend({
                'loaded': $gallery.find('.main-swiper .swiper-slide:not(.swiper-slide-duplicate)').length,
                'refresh': false,
            }, $gallery.data());
        };

        let $swiper = $('.swiper-tv');

        if ($swiper.length === 0) {
            return;
        }

        cvJsRequire.require(['swiper-bundle.js', 'swiper-bundle.css'], function () {

            let delay = $.urlParam('delay') || 6,
                effect = $.urlParam('effect') || 3;

            delay *= 1000;
            effect = effects[effect % effects.length];

            const swiper = new Swiper('.swiper-tv', {
                loop: true,
                autoplay: {
                    delay: delay,
                    disableOnInteraction: false,
                    pauseOnMouseEnter: false,
                    reverseDirection: false,
                    stopOnLastSlide: false,
                    waitForTransition: true
                },
                effect: effect,
                coverflowEffect: {
                    rotate: 90,
                    slideShadows: false,
                    depth: 100,
                    modifier: 1,
                    scale: 1,
                    stretch: 0
                },
                cardsEffect: {
                    perSlideOffset: 8,
                    perSlideRotate: 2,
                    rotate: true,
                    slideShadows: false
                },
                cubeEffect: {
                    slideShadows: false,
                    shadow: true,
                    shadowOffset: 20,
                    shadowScale: 0.94
                },
                flipEffect: {
                    slideShadows: false,
                    limitRotation: true,
                },
                fadeEffect: {
                    crossFade: true
                },
                navigation: false,
                pagination: false,
                preloadImages: false,
                lazy: true,
                autoHeight: true,
                on: {
                    init: slideChange,
                    slideChangeTransitionEnd: slideChange,
                    slideChangeTransitionStart : () => {
                        if (typeof videojs === 'undefined') {
                            return;
                        }

                        videojs.getAllPlayers().forEach((player) => {

                            if (player.paused() === true) {
                                return;
                            }

                            player.pause();
                            this.autoplay.start();
                        });
                    }
                },
            });

            $swiper.trigger('init.swiper');
        });
    });

})(jQuery);

//# sourceMappingURL=maps/swiper-tv.min.js.map
