(function ($) {

    'use strict';
    var $document = $(document),
        on = function (event, selector, fn) {
            $document.on(event, selector, fn);
        };

    window.cvJsRequire = window.cvJsRequire || {
        'require': function () {
            console.error('cvJsRequire is required for this to work!')
        }
    }

    $(function () {

        const videoClick = function (swiper) {

            const $currentSlide = this.closest('.swiper-slide'),
                  $swiper = $(swiper.el);

            swiper.enable();

            if ($currentSlide.find('video').length === 0) {
                return;
            }

            let $video = $currentSlide.find('video'),
                $videoContainer = $currentSlide.find('.video-modal-content');

            if ($videoContainer.hasClass('video-js-initialized') === true) {
                const player = videojs.getPlayer($video[0]);
                player.currentTime(0);
                player.play();
                return;
            }

            if ($videoContainer.hasClass('video-js-attached') === true) {
                return;
            }

            $videoContainer.addClass('video-js-attached');

            cvJsRequire.require(['video.js', 'video-js.css'], function () {

                const player = videojs($video[0], {
                    controls: true,
                    autoplay: 'any',
                    preload: 'metadata',
                    fluid: true,
                });

                player.on('ended', function () {
                    player.currentTime(0);
                    swiper.enable();
                    $currentSlide.removeClass('video-playing');
                });

                player.on('play', function () {
                    swiper.disable();
                    $currentSlide.addClass('video-playing');
                });

                player.on('pause', function () {
                    swiper.enable();
                });

                player.on('ready', function () {
                    $videoContainer.addClass('video-js-initialized');
                    _.delay(() => {
                        swiper.update();
                        player.muted(true);
                        player.play();
                    }, 100);
                });
            });
        };

        const requestNewImages = function (dataCallback, successCallback, failCallback) {

            if ($document.data('ajax.requestNewImages.in_progess') === true) {
                return;
            }

            $document.data('ajax.requestNewImages.in_progess', true);

            $.ajax({
                type: 'POST',
                cache: false,
                url: pg_data.ajaxurl,
                data: {
                    'form': dataCallback(),
                    'action': 'pg-next-page',
                    'csrf': pg_data['pg-next-page']
                }
            })
                .done(successCallback)
                .fail(failCallback)
                .complete(() => {
                    $document.data('ajax.requestNewImages.in_progess', false);
                })
        };

        const dataCallback = function () {
            const $gallery = $('div.the-gallery.photobooth-gallery-swiper');

            return $.extend({
                'loaded': $gallery.find('.grid-swiper .swiper-slide:not(.swiper-slide-duplicate)').length,
                'refresh': false,
            }, $gallery.data());
        };

        cvJsRequire.require(['swiper-bundle.js', 'swiper-bundle.css'], function () {

            const reloadImages = (swiper) => {
                const $gallery = $('div.photobooth-gallery-swiper');

                let dataSet = dataCallback(),
                    total = parseInt($gallery.data('total')),
                    currentSlide = swiper.realIndex + 1,
                    reloadAt = Math.max(1,dataSet.loaded - 3);

                if (total === dataSet.loaded) {
                    return;
                }

                if (currentSlide < reloadAt) {
                    return;
                }

                requestNewImages(dataCallback, (response) => {
                    if (response.error === true) {
                        return false;
                    }

                    if (response.hasElements === false) {
                        return true;
                    }

                    const $content = $(response.data),
                        $contentGallery = $content.find('div.the-gallery'),
                        $gallery = $('div.photobooth-gallery-swiper');

                    let $slides = $contentGallery.find('.swiper.grid-swiper .swiper-slide');

                    $slides = $slides.filter((index, item) => {
                        let $item = $(item);

                        return $('#' + $item.attr('id')).length === 0;
                    });

                    if ($slides.length === 0) {
                        return true;
                    }

                    $gallery.data('page', $contentGallery.data('page'));
                    $gallery.data('total', $contentGallery.data('total'));

                    swiper.appendSlide($slides);
                    swiper.update();

                    let $images = $(swiper.slides).find('img.zoomable-image');

                    const zoom = $document.data('pbg-zoom') || false;

                    if (zoom) {
                        zoom.attach($images.map((idx, item) => item).toArray());
                    }

                    return true;
                }, () => {});
            };

            const gridSwiper = new Swiper(".grid-swiper", {
                direction: 'vertical',
                slidesPerView: 3,
                spaceBetween: 30,
                preloadImages: true,
                speed: 800,
                freeMode: true,
                mousewheelControl: true,
                watchSlidesProgress: true,
                mousewheel: {
                    releaseOnEdges: true,
                },
                pagination: {
                    el: '.swiper-pagination',
                    clickable: true,
                    dynamicBullets: true,
                    dynamicMainBullets: 10,
                    type: 'bullets',
                    renderBullet: function (index, className) {
                        index += 1;
                        const number = (index < 10 ? '0' : '') + index;

                        return '<span class="' + className + '">' + number + '</span>';
                    }
                },
                on: {
                    init: (swiper) => {
                        let totalGap = swiper.passedParams.spaceBetween * (swiper.passedParams.slidesPerView - 1);
                        let containerHeight = swiper.passedParams.slidesPerView * swiper.slides[0].clientHeight + totalGap;
                        swiper.el.style.height = containerHeight + 'px';
                    },
                    slideChange: (swiper) => {
                        reloadImages(swiper);
                        let $swiper = $(swiper.el);

                        if (typeof videojs === 'undefined') {
                            return;
                        }

                        $swiper.find('.video-playing').removeClass('video-playing');

                        videojs.getAllPlayers().forEach((player) => {

                            if (player.paused() === true) {
                                return;
                            }

                            player.pause();
                        });
                    }
                }
            });
        });

        on('show.bs.modal', (ev) => {
            let $modal = $(ev.target),
                $socialContainer = $(ev.relatedTarget).closest('.social-media-container');

            if ($socialContainer.length === 0) {
                return;
            }

            const swiper = $socialContainer.closest('.swiper')[0].swiper;

            $modal
                .data('smc-id', $socialContainer.attr('id'))
                .find('.modal-body').html($socialContainer.find('.copy-content').html());
            $socialContainer.addClass('pg-visible');
            swiper.disable();
        });

        on('hide.bs.modal', (ev) => {
            let $modal = $(ev.target),
                $socialContainer = $('#' + $modal.data('smc-id'));

            if ($socialContainer.length === 0) {
                return;
            }

            const swiper = $socialContainer.closest('.swiper')[0].swiper;

            $socialContainer.removeClass('pg-visible');
            swiper.enable();
        });

        on('click', 'a.video-loader', (ev) => {
            ev.preventDefault();
            let $this = $(ev.target),
                swiper = $this.closest('.grid-swiper')[0].swiper;
            videoClick.bind($this)(swiper);
        });

        on('pg.admin-remove-images', (ev, data) => {
            let $container = $(ev.target),
                swiper = $container.find('.grid-swiper')[0].swiper;

            let items = data.items.map((idx, item) => {
                return $(item).closest('.swiper-slide').index();
            }).toArray();

            swiper.removeSlide(items);
        });

        if ($('.zoomable-image').length > 0) {
            cvJsRequire.require('zoom.min.js', () => {
                $document.data('pbg-zoom', Zoom(".zoomable-image", {
                    background: "auto"
                }));
            });
        }
    });

})(jQuery);

//# sourceMappingURL=maps/swiper-vertical.min.js.map
