<?php

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly.
}

use Elementor\Controls_Manager;
use Elementor\Group_Control_Typography;
use Elementor\Core\Schemes\Color;
use Elementor\Core\Schemes\Typography;
use Elementor\Widget_Counter;
use Photobooth\SocialMedia;
use Photobooth\Statistics\SocialMediaStatistics;
use Photobooth\StringTemplate\Engine;

/**
 * Created by PhpStorm.
 * User: b2wsra
 * Date: 23.11.2018
 * Time: 10:28
 */
class Photobooth_Counter_Widget extends Widget_Counter {


    /**
     * @var array
     */
    private $data_types;

    /**
     * Photobooth_Counter_Widget constructor.
     *
     * @param array $data
     * @param null $args
     *
     * @throws Exception
     */
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        $this->data_types = [
            '_all'                  => __( 'All [Social Media]', 'photobooth-gallery' ),

            // Social Media data
            'download'              => __( 'Downloads [Social Media]', 'photobooth-gallery' ),
            'download-image'        => __( 'Download Images [Social Media]', 'photobooth-gallery' ),
            'download-video'        => __( 'Download Videos [Social Media]', 'photobooth-gallery' ),
            'e-mail'                => __( 'E-Mail [Social Media]', 'photobooth-gallery' ),
            'twitter'               => __( 'Twitter [Social Media]', 'photobooth-gallery' ),
            'facebook'              => __( 'Facebook [Social Media]', 'photobooth-gallery' ),
            'fb-messenger'          => __( 'Facebook Messenger [Social Media]', 'photobooth-gallery' ),
            'g-plus'                => __( 'Google Plus [Social Media]', 'photobooth-gallery' ),
            'linked-in'             => __( 'Linked-In [Social Media]', 'photobooth-gallery' ),
            'pinterest'             => __( 'Pinterest [Social Media]', 'photobooth-gallery' ),
            'whatsapp'              => __( 'Whatsapp [Social Media]', 'photobooth-gallery' ),
            'xing'                  => __( 'Xing [Social Media]', 'photobooth-gallery' ),
            'wechat'                => __( 'WeChat [Social Media]', 'photobooth-gallery' ),

            // Website Data
            'all-viewed'            => __( 'All views [Gallery]', 'photobooth-gallery' ),
            'image-preview-clicked' => __( 'Photos viewed [Gallery]', 'photobooth-gallery' ),
            'video-preview-clicked' => __( 'Videos viewed [Gallery]', 'photobooth-gallery' ),

            // XML Data
            'xml-prints'            => __( 'Photos printed [XML]', 'photobooth-gallery' ),
            'xml-sessions'          => __( 'Sessions [XML]', 'photobooth-gallery' ),
            'xml-photos'            => __( 'Photos taken [XML]', 'photobooth-gallery' ),

            // Wordpress Data
            'wp-image-imported'     => __( 'Photos uploaded [Wordpress]', 'photobooth-gallery' ),
            'wp-video-imported'     => __( 'Videos uploaded [Wordpress]', 'photobooth-gallery' ),

        ];
    }

    public function get_categories() {

        return [ 'pg-photobooth-gallery' ];
    }

    public function get_icon() {

        return 'pg counter';
    }

    public function get_name() {

        return 'pg-counter';
    }

    protected function get_html_wrapper_class() {

        return 'elementor-widget-counter';
    }


    protected function register_controls() {

        $this->start_controls_section( 'section_counter',
            [
                'label' => __( 'Counter', 'elementor' ),
            ] );

        $this->add_control( 'view_selector',
            [
                'label'   => __( 'View Type', 'photobooth-gallery' ),
                'type'    => Controls_Manager::SELECT,
                'default' => SocialMedia::SHARE,
                'options' => [
                    SocialMedia::SHARE => __( 'Shared', 'photobooth-gallery' ),
                    SocialMedia::VIEW  => __( 'Viewed', 'photobooth-gallery' ),
                    'unique'                       => __( 'Unique', 'photobooth-gallery' ),
                    'average'                      => __( 'Average', 'photobooth-gallery' ),
                ],
            ] );


        $this->add_control( 'data_selector',
            [
                'label'   => __( 'Data Type', 'photobooth-gallery' ),
                'type'    => Controls_Manager::SELECT,
                'default' => '_all',
                'options' => $this->data_types,
            ] );

        $this->add_control( 'auto_hide',
            [
                'label'   => __( 'Auto Hide on empty values', 'photobooth-gallery' ),
                'type'    => Controls_Manager::SELECT,
                'default' => 'no',
                'options' => [
                    'yes' => __( 'Yes', 'photobooth-gallery' ),
                    'no'  => __( 'No', 'photobooth-gallery' ),
                ],
            ] );


        $this->add_control( 'ending_number',
            [
                'label'   => __( 'Ending Number', 'elementor' ),
                'type'    => Controls_Manager::HIDDEN,
                'default' => 100,
            ] );

        $this->add_control( 'prefix',
            [
                'label'       => __( 'Number Prefix', 'elementor' ),
                'type'        => Controls_Manager::TEXT,
                'default'     => '',
                'placeholder' => 1,
            ] );

        $this->add_control( 'suffix',
            [
                'label'       => __( 'Number Suffix', 'elementor' ),
                'type'        => Controls_Manager::TEXT,
                'default'     => '',
                'placeholder' => __( 'Plus', 'elementor' ),
            ] );

        $this->add_control( 'duration',
            [
                'label'   => __( 'Animation Duration', 'elementor' ),
                'type'    => Controls_Manager::NUMBER,
                'default' => 2000,
                'min'     => 100,
                'step'    => 100,
            ] );

        $this->add_control( 'thousand_separator',
            [
                'label'     => __( 'Thousand Separator', 'elementor' ),
                'type'      => Controls_Manager::SWITCHER,
                'default'   => 'yes',
                'label_on'  => __( 'Show', 'elementor' ),
                'label_off' => __( 'Hide', 'elementor' ),
            ] );

        $this->add_control( 'thousand_separator_char',
            [
                'label'     => __( 'Separator', 'elementor' ),
                'type'      => Controls_Manager::SELECT,
                'condition' => [
                    'thousand_separator' => 'yes',
                ],
                'options'   => [
                    ''  => 'Default',
                    '.' => 'Dot',
                    ' ' => 'Space',
                ],
            ] );

        $this->add_control( 'title',
            [
                'label'       => __( 'Title', 'elementor' ),
                'type'        => Controls_Manager::TEXT,
                'label_block' => true,
                'default'     => __( 'Cool Number', 'elementor' ),
                'placeholder' => __( 'Cool Number', 'elementor' ),
            ] );

        $this->add_control( 'view',
            [
                'label'   => __( 'View', 'elementor' ),
                'type'    => Controls_Manager::HIDDEN,
                'default' => 'traditional',
            ] );

        $this->end_controls_section();

        $this->start_controls_section( 'section_number',
            [
                'label' => __( 'Number', 'elementor' ),
                'tab'   => Controls_Manager::TAB_STYLE,
            ] );

        $this->add_control( 'number_color',
            [
                'label'     => __( 'Text Color', 'elementor' ),
                'type'      => Controls_Manager::COLOR,
                'scheme'    => [
                    'type'  => Color::get_type(),
                    'value' => Color::COLOR_1,
                ],
                'selectors' => [
                    '{{WRAPPER}} .elementor-counter-number-wrapper' => 'color: {{VALUE}};',
                ],
            ] );

        $this->add_group_control( Group_Control_Typography::get_type(),
            [
                'name'     => 'typography_number',
                'scheme'   => Typography::TYPOGRAPHY_1,
                'selector' => '{{WRAPPER}} .elementor-counter-number-wrapper',
            ] );

        $this->end_controls_section();

        $this->start_controls_section( 'section_title',
            [
                'label' => __( 'Title', 'elementor' ),
                'tab'   => Controls_Manager::TAB_STYLE,
            ] );

        $this->add_control( 'title_color',
            [
                'label'     => __( 'Text Color', 'elementor' ),
                'type'      => Controls_Manager::COLOR,
                'scheme'    => [
                    'type'  => Color::get_type(),
                    'value' => Color::COLOR_2,
                ],
                'selectors' => [
                    '{{WRAPPER}} .elementor-counter-title' => 'color: {{VALUE}};',
                ],
            ] );

        $this->add_group_control( Group_Control_Typography::get_type(),
            [
                'name'     => 'typography_title',
                'scheme'   => Typography::TYPOGRAPHY_2,
                'selector' => '{{WRAPPER}} .elementor-counter-title',
            ] );

        $this->end_controls_section();
    }

    private function get_value_from_service( $settings ) {

        $service = new SocialMediaStatistics();
        $engine  = new Engine();

        $service->setCode( get_query_var( 'pg-code' ) );

        $number = $service->count_by_network( $settings[ 'data_selector' ], $settings[ 'view_selector' ] );

        $title = $this->data_types[ $settings[ 'data_selector' ] ] ?? __( 'N/A', 'photobooth-gallery' );

        $title = explode( '[', $title );


        $title = $engine->render( $settings[ 'title' ],
            [
                'network' => reset( $title ),
            ] );

        return compact( 'number', 'title' );
    }

    protected function render() {

        $settings = $this->get_settings_for_display();

        $data = $this->get_value_from_service( $settings );

        $settings[ 'ending_number' ] = $data[ 'number' ];

        if ( $data[ 'number' ] <= 0 && $settings[ 'auto_hide' ] === 'yes' ) {
            return;
        }

        $this->add_render_attribute( 'counter',
            [
                'class'         => 'elementor-counter-number',
                'data-duration' => $settings[ 'duration' ],
                'data-to-value' => $settings[ 'ending_number' ],
            ] );

        if ( ! empty( $settings[ 'thousand_separator' ] ) ) {
            $delimiter = empty( $settings[ 'thousand_separator_char' ] ) ? ',' : $settings[ 'thousand_separator_char' ];
            $this->add_render_attribute( 'counter', 'data-delimiter', $delimiter );
        }

        $this->add_render_attribute( '_wrapper',
            [
                'data-autohide' => true,
                'data-value'    => $settings[ 'ending_number' ],
            ] );

        ?>
        <div class="elementor-counter">
            <div class="elementor-counter-number-wrapper">
                <span class="elementor-counter-number-prefix"><?php echo $settings[ 'prefix' ]; ?></span>
                <span <?php echo $this->get_render_attribute_string( 'counter' ); ?>><?php echo $settings[ 'ending_number' ] ?></span>
                <span class="elementor-counter-number-suffix"><?php echo $settings[ 'suffix' ]; ?></span>
            </div>
            <?php if ( $settings[ 'title' ] ) : ?>
                <div class="elementor-counter-title"><?php echo $data[ 'title' ]; ?></div>
            <?php endif; ?>
        </div>
        <?php
    }


}
