<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

use Elementor\Controls_Manager;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Css_Filter;
use Elementor\Group_Control_Image_Size;
use Elementor\Group_Control_Typography;
use Elementor\Plugin;
use Elementor\Core\Schemes\Color;
use Elementor\Core\Schemes\Typography;
use Elementor\Widget_Image;
use Photobooth\Helper;
use Photobooth\Statistics\SocialMediaStatistics;
use Photobooth\StringTemplate\Engine;

/**
 * Created by PhpStorm.
 * User: b2wsra
 * Date: 23.11.2018
 * Time: 10:28
 */
class Photobooth_Image_Widget extends Widget_Image {

	public function get_categories() {
		return [ 'pg-photobooth-gallery' ];
	}

	public function get_icon() {
		return 'pg image';
	}

	public function get_name() {
		return 'pg-image';
	}

	protected function get_html_wrapper_class() {
		return 'elementor-widget-image';
	}

	/**
	 * Register image widget controls.
	 *
	 * Adds different input fields to allow the user to change and customize the widget settings.
	 *
	 * @since 1.0.0
	 * @access protected
	 */
	protected function register_controls() {
		$this->start_controls_section( 'section_image', [
			'label' => __( 'Image', 'elementor' ),
		] );

		$this->add_control( 'image_type', [
			'label'   => __( 'Social Media Type', 'photobooth-gallery' ),
			'type'    => Controls_Manager::SELECT,
			'default' => 'total_shares',
			'options' => [
				\Photobooth\SocialMedia::SHARE => __( 'Most shared Image', 'photobooth-gallery' ),
				\Photobooth\SocialMedia::VIEW  => __( 'Most viewed Image', 'photobooth-gallery' ),
			],
		] );

		$this->add_control( 'image', [
				'label'   => __( 'Choose Fallback Image', 'photobooth-gallery' ),
				'type'    => Controls_Manager::MEDIA,
				'dynamic' => [
					'active' => true,
				],
				'default' => [
					'url' => \Elementor\Utils::get_placeholder_image_src(),
				],
			] );


		$this->add_group_control( Group_Control_Image_Size::get_type(), [
			'name'      => 'image',
			// Usage: `{name}_size` and `{name}_custom_dimension`, in this case `image_size` and `image_custom_dimension`.
			'default'   => 'large',
			'separator' => 'none',
		] );

		$this->add_responsive_control( 'align', [
			'label'     => __( 'Alignment', 'elementor' ),
			'type'      => Controls_Manager::CHOOSE,
			'options'   => [
				'left'   => [
					'title' => __( 'Left', 'elementor' ),
					'icon'  => 'fa fa-align-left',
				],
				'center' => [
					'title' => __( 'Center', 'elementor' ),
					'icon'  => 'fa fa-align-center',
				],
				'right'  => [
					'title' => __( 'Right', 'elementor' ),
					'icon'  => 'fa fa-align-right',
				],
			],
			'selectors' => [
				'{{WRAPPER}}' => 'text-align: {{VALUE}};',
			],
		] );

		$this->add_control( 'caption', [
			'label'       => __( 'Caption', 'elementor' ),
			'type'        => Controls_Manager::TEXT,
			'default'     => '',
			'placeholder' => __( 'Enter your image caption', 'elementor' ),
			'dynamic'     => [
				'active' => true,
			],
		] );

		$this->add_control( 'link_to', [
			'label'   => __( 'Link to', 'elementor' ),
			'type'    => Controls_Manager::SELECT,
			'default' => 'none',
			'options' => [
				'none'   => __( 'None', 'elementor' ),
				'file'   => __( 'Media File', 'elementor' ),
				'custom' => __( 'Custom URL', 'elementor' ),
			],
		] );

		$this->add_control( 'link', [
			'label'       => __( 'Link to', 'elementor' ),
			'type'        => Controls_Manager::URL,
			'dynamic'     => [
				'active' => true,
			],
			'placeholder' => __( 'https://your-link.com', 'elementor' ),
			'condition'   => [
				'link_to' => 'custom',
			],
			'show_label'  => false,
		] );

		$this->add_control( 'open_lightbox', [
			'label'     => __( 'Lightbox', 'elementor' ),
			'type'      => Controls_Manager::SELECT,
			'default'   => 'default',
			'options'   => [
				'default' => __( 'Default', 'elementor' ),
				'yes'     => __( 'Yes', 'elementor' ),
				'no'      => __( 'No', 'elementor' ),
			],
			'condition' => [
				'link_to' => 'file',
			],
		] );

		$this->add_control( 'view', [
			'label'   => __( 'View', 'elementor' ),
			'type'    => Controls_Manager::HIDDEN,
			'default' => 'traditional',
		] );

		$this->end_controls_section();

		$this->start_controls_section( 'section_style_image', [
			'label' => __( 'Image', 'elementor' ),
			'tab'   => Controls_Manager::TAB_STYLE,
		] );

		$this->add_responsive_control( 'width', [
			'label'          => __( 'Width', 'elementor' ),
			'type'           => Controls_Manager::SLIDER,
			'default'        => [
				'unit' => '%',
			],
			'tablet_default' => [
				'unit' => '%',
			],
			'mobile_default' => [
				'unit' => '%',
			],
			'size_units'     => [ '%', 'px', 'vw' ],
			'range'          => [
				'%'  => [
					'min' => 1,
					'max' => 100,
				],
				'px' => [
					'min' => 1,
					'max' => 1000,
				],
				'vw' => [
					'min' => 1,
					'max' => 100,
				],
			],
			'selectors'      => [
				'{{WRAPPER}} .elementor-image img' => 'width: {{SIZE}}{{UNIT}};',
			],
		] );

		$this->add_responsive_control( 'space', [
			'label'          => __( 'Max Width', 'elementor' ) . ' (%)',
			'type'           => Controls_Manager::SLIDER,
			'default'        => [
				'unit' => '%',
			],
			'tablet_default' => [
				'unit' => '%',
			],
			'mobile_default' => [
				'unit' => '%',
			],
			'size_units'     => [ '%' ],
			'range'          => [
				'%' => [
					'min' => 1,
					'max' => 100,
				],
			],
			'selectors'      => [
				'{{WRAPPER}} .elementor-image img' => 'max-width: {{SIZE}}{{UNIT}};',
			],
		] );

		$this->add_control( 'separator_panel_style', [
			'type'  => Controls_Manager::DIVIDER,
			'style' => 'thick',
		] );

		$this->start_controls_tabs( 'image_effects' );

		$this->start_controls_tab( 'normal', [
			'label' => __( 'Normal', 'elementor' ),
		] );

		$this->add_control( 'opacity', [
			'label'     => __( 'Opacity', 'elementor' ),
			'type'      => Controls_Manager::SLIDER,
			'range'     => [
				'px' => [
					'max'  => 1,
					'min'  => 0.10,
					'step' => 0.01,
				],
			],
			'selectors' => [
				'{{WRAPPER}} .elementor-image img' => 'opacity: {{SIZE}};',
			],
		] );

		$this->add_group_control( Group_Control_Css_Filter::get_type(), [
			'name'     => 'css_filters',
			'selector' => '{{WRAPPER}} .elementor-image img',
		] );

		$this->end_controls_tab();

		$this->start_controls_tab( 'hover', [
			'label' => __( 'Hover', 'elementor' ),
		] );

		$this->add_control( 'opacity_hover', [
			'label'     => __( 'Opacity', 'elementor' ),
			'type'      => Controls_Manager::SLIDER,
			'range'     => [
				'px' => [
					'max'  => 1,
					'min'  => 0.10,
					'step' => 0.01,
				],
			],
			'selectors' => [
				'{{WRAPPER}} .elementor-image:hover img' => 'opacity: {{SIZE}};',
			],
		] );

		$this->add_group_control( Group_Control_Css_Filter::get_type(), [
			'name'     => 'css_filters_hover',
			'selector' => '{{WRAPPER}} .elementor-image:hover img',
		] );

		$this->add_control( 'background_hover_transition', [
			'label'     => __( 'Transition Duration', 'elementor' ),
			'type'      => Controls_Manager::SLIDER,
			'range'     => [
				'px' => [
					'max'  => 3,
					'step' => 0.1,
				],
			],
			'selectors' => [
				'{{WRAPPER}} .elementor-image img' => 'transition-duration: {{SIZE}}s',
			],
		] );

		$this->add_control( 'hover_animation', [
			'label' => __( 'Hover Animation', 'elementor' ),
			'type'  => Controls_Manager::HOVER_ANIMATION,
		] );

		$this->end_controls_tab();

		$this->end_controls_tabs();

		$this->add_group_control( Group_Control_Border::get_type(), [
			'name'      => 'image_border',
			'selector'  => '{{WRAPPER}} .elementor-image img',
			'separator' => 'before',
		] );

		$this->add_responsive_control( 'image_border_radius', [
			'label'      => __( 'Border Radius', 'elementor' ),
			'type'       => Controls_Manager::DIMENSIONS,
			'size_units' => [ 'px', '%' ],
			'selectors'  => [
				'{{WRAPPER}} .elementor-image img' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
			],
		] );

		$this->add_group_control( Group_Control_Box_Shadow::get_type(), [
			'name'     => 'image_box_shadow',
			'exclude'  => [
				'box_shadow_position',
			],
			'selector' => '{{WRAPPER}} .elementor-image img',
		] );

		$this->end_controls_section();

		$this->start_controls_section( 'section_style_caption', [
			'label'     => __( 'Caption', 'elementor' ),
			'tab'       => Controls_Manager::TAB_STYLE,
			'condition' => [
				'caption!' => '',
			],
		] );

		$this->add_control( 'caption_align', [
			'label'     => __( 'Alignment', 'elementor' ),
			'type'      => Controls_Manager::CHOOSE,
			'options'   => [
				'left'    => [
					'title' => __( 'Left', 'elementor' ),
					'icon'  => 'fa fa-align-left',
				],
				'center'  => [
					'title' => __( 'Center', 'elementor' ),
					'icon'  => 'fa fa-align-center',
				],
				'right'   => [
					'title' => __( 'Right', 'elementor' ),
					'icon'  => 'fa fa-align-right',
				],
				'justify' => [
					'title' => __( 'Justified', 'elementor' ),
					'icon'  => 'fa fa-align-justify',
				],
			],
			'default'   => '',
			'selectors' => [
				'{{WRAPPER}} .widget-image-caption' => 'text-align: {{VALUE}};',
			],
		] );

		$this->add_control( 'text_color', [
			'label'     => __( 'Text Color', 'elementor' ),
			'type'      => Controls_Manager::COLOR,
			'default'   => '',
			'selectors' => [
				'{{WRAPPER}} .widget-image-caption' => 'color: {{VALUE}};',
			],
			'scheme'    => [
				'type'  => Color::get_type(),
				'value' => Color::COLOR_3,
			],
		] );

		$this->add_group_control( Group_Control_Typography::get_type(), [
			'name'     => 'caption_typography',
			'selector' => '{{WRAPPER}} .widget-image-caption',
			'scheme'   => Typography::TYPOGRAPHY_3,
		] );

		$this->add_responsive_control( 'caption_space', [
			'label'     => __( 'Spacing', 'elementor' ),
			'type'      => Controls_Manager::SLIDER,
			'range'     => [
				'px' => [
					'min' => 0,
					'max' => 100,
				],
			],
			'selectors' => [
				'{{WRAPPER}} .widget-image-caption' => 'margin-top: {{SIZE}}{{UNIT}};',
			],
		] );

		$this->end_controls_section();
	}

	/**
	 * Check if the current widget has caption
	 *
	 * @access private
	 * @since 2.3.0
	 *
	 * @param array $settings
	 *
	 * @return boolean
	 */
	private function has_caption( $settings ) {
		return ! empty( $settings['caption'] );
	}

	/**
	 * Get the caption for current widget.
	 *
	 * @access private
	 * @since 2.3.0
	 *
	 * @param $settings
	 *
	 * @return string
	 */
	private function get_caption( $settings ) {

	    if ( empty( $settings['caption'] ) ) {
	        return '';
        }

		$engine  = new Engine();

	    return $engine->render( $settings['caption'], [
	        'size' => $settings['images-count'] ?? ''
        ] );
	}

	private function get_value_from_service( $settings ) {

		$service = new SocialMediaStatistics();
		$service->setCode( get_query_var( 'pg-code' ) );

		$db_data = $service->top_image( $settings['image_type'] );

		if ( $db_data === 0 ) {
			return $settings;
		}

		list( $size, $attachment_id ) = $db_data;

		$image_data = wp_get_attachment_image_src( $attachment_id, 'pg-preview' );

		$settings['images-count'] = $size;

		if ( $image_data === false ) {
			Helper::log( sprintf( '[ Statistic Image Widget ] Image with ID %d could not be found in Media Library, maybe it has been deleted?', $attachment_id ), 'error' );

			return $settings;
		}

		$settings['image'] = [
			'url' => reset( $image_data ),
			'id'  => $attachment_id
		];

		return $settings;
	}

	/**
	 * Render image widget output on the frontend.
	 *
	 * Written in PHP and used to generate the final HTML.
	 *
	 * @since 1.0.0
	 * @access protected
	 */
	protected function render() {
		$settings = $this->get_settings_for_display();

		$settings = $this->get_value_from_service( $settings );

		if ( empty( $settings['image']['url'] ) ) {
			return;
		}

		$has_caption = $this->has_caption( $settings );

		$this->add_render_attribute( 'wrapper', 'class', 'elementor-image' );

		if ( ! empty( $settings['shape'] ) ) {
			$this->add_render_attribute( 'wrapper', 'class', 'elementor-image-shape-' . $settings['shape'] );
		}

		$link = false;

		if ( $link ) {
			$this->add_render_attribute( 'link', [
				'href'                         => $link['url'],
				'data-elementor-open-lightbox' => $settings['open_lightbox'],
			] );

			if ( Plugin::$instance->editor->is_edit_mode() ) {
				$this->add_render_attribute( 'link', [
					'class' => 'elementor-clickable',
				] );
			}

			if ( ! empty( $link['is_external'] ) ) {
				$this->add_render_attribute( 'link', 'target', '_blank' );
			}

			if ( ! empty( $link['nofollow'] ) ) {
				$this->add_render_attribute( 'link', 'rel', 'nofollow' );
			}
		} ?>
        <div <?php echo $this->get_render_attribute_string( 'wrapper' ); ?>>
			<?php if ( $has_caption ) : ?>
            <figure class="wp-caption">
				<?php endif; ?>
				<?php if ( $link ) : ?>
                <a <?php echo $this->get_render_attribute_string( 'link' ); ?>>
					<?php endif; ?>
					<?php echo Group_Control_Image_Size::get_attachment_image_html( $settings ); ?>
					<?php if ( $link ) : ?>
                </a>
			<?php endif; ?>
				<?php if ( $has_caption ) : ?>
                    <figcaption
                            class="widget-image-caption wp-caption-text"><?php echo $this->get_caption( $settings ); ?></figcaption>
				<?php endif; ?>
				<?php if ( $has_caption ) : ?>
            </figure>
		<?php endif; ?>
        </div>
		<?php
	}

	/**
	 * Render image widget output in the editor.
	 *
	 * Written as a Backbone JavaScript template and used to generate the live preview.
	 *
	 * @since 1.0.0
	 * @access protected
	 */
	protected function _content_template() {
		?>
        <# if ( settings.image.url ) {
        var image = {
        id: settings.image.id,
        url: settings.image.url,
        size: settings.image_size,
        dimension: settings.image_custom_dimension,
        model: view.getEditModel()
        };

        var image_url = elementor.imagesManager.getImageUrl( image );

        if ( ! image_url ) {
        return;
        }

        var hasCaption = function() {
        return settings.caption
        }

        var ensureAttachmentData = function( id ) {
        if ( 'undefined' === typeof wp.media.attachment( id ).get( 'caption' ) ) {
        wp.media.attachment( id ).fetch().then( function( data ) {
        view.render();
        } );
        }
        }

        var getAttachmentCaption = function( id ) {
        if ( ! id ) {
        return '';
        }
        ensureAttachmentData( id );
        return wp.media.attachment( id ).get( 'caption' );
        }

        var getCaption = function() {
        if ( ! hasCaption() ) {
        return '';
        }
        return settings.caption;
        }

        var link_url;

        if ( 'custom' === settings.link_to ) {
        link_url = settings.link.url;
        }

        if ( 'file' === settings.link_to ) {
        link_url = settings.image.url;
        }

        #>
        <div class="elementor-image{{ settings.shape ? ' elementor-image-shape-' + settings.shape : '' }}"><#
            var imgClass = '';

            if ( '' !== settings.hover_animation ) {
            imgClass = 'elementor-animation-' + settings.hover_animation;
            }

            if ( hasCaption() ) {
            #>
            <figure class="wp-caption"><#
                }

                if ( link_url ) {
                #><a class="elementor-clickable" data-elementor-open-lightbox="{{ settings.open_lightbox }}"
                     href="{{ link_url }}"><#
                    }
                    #><img src="{{ image_url }}" class="{{ imgClass }}"/><#

                    if ( link_url ) {
                    #></a><#
                }

                if ( hasCaption() ) {
                #>
                <figcaption class="widget-image-caption wp-caption-text">{{{ getCaption() }}}</figcaption>
                <#
                }

                if ( hasCaption() ) {
                #>
            </figure>
            <#
            }

            #>
        </div><#
        } #>
		<?php
	}

}
