<?php

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

use Elementor\Controls_Manager;
use Elementor\Core\Kits\Documents\Tabs\Global_Colors;
use Elementor\Core\Kits\Documents\Tabs\Global_Typography;
use Elementor\Group_Control_Text_Shadow;
use Elementor\Group_Control_Typography;
use Elementor\Widget_Progress;
use Photobooth\SocialMedia;
use Photobooth\Statistics\SocialMediaStatistics;
use Photobooth\StringTemplate\Engine;

class Photobooth_Progressbar_Widget extends Widget_Progress
{

    /**
     * @var array
     */
    private $data_types;

    /**
     * Photobooth_Progressbar_Widget constructor.
     *
     * @param array $data
     * @param null $args
     *
     * @throws Exception
     */
    public function __construct($data = [], $args = null)
    {
        parent::__construct($data, $args);

        $this->data_types = [
            '_all' => __('All', 'photobooth-gallery'),

            // Social Media data
            'download' => __('Download', 'photobooth-gallery'),
            'e-mail' => __('E-Mail', 'photobooth-gallery'),
            'twitter' => __('Twitter', 'photobooth-gallery'),
            'facebook' => __('Facebook', 'photobooth-gallery'),
            'fb-messenger' => __('Facebook Messenger', 'photobooth-gallery'),
            'g-plus' => __('Google Plus', 'photobooth-gallery'),
            'linked-in' => __('Linked-In', 'photobooth-gallery'),
            'pinterest' => __('Pinterest', 'photobooth-gallery'),
            'whatsapp' => __('Whatsapp', 'photobooth-gallery'),
            'xing' => __('Xing', 'photobooth-gallery'),
            'wechat' => __('WeChat', 'photobooth-gallery'),

            // Website Data
            'all-viewed' => __('All views', 'photobooth-gallery'),
            'image-preview-clicked' => __('Photos viewed', 'photobooth-gallery'),
            'video-preview-clicked' => __('Videos viewed', 'photobooth-gallery'),

            // XML Data
            'xml-prints' => __('Photos printed', 'photobooth-gallery'),
            'xml-sessions' => __('Sessions', 'photobooth-gallery'),
            'xml-photos' => __('Photos taken', 'photobooth-gallery'),
        ];
    }

    public function get_categories()
    {
        return ['pg-photobooth-gallery'];
    }

    public function get_icon()
    {
        return 'pg progressbar';
    }

    public function get_name()
    {
        return 'pg-progressbar';
    }

    protected function get_html_wrapper_class()
    {
        return 'elementor-widget-progress';
    }

    /**
     * Register progress widget controls.
     *
     * Adds different input fields to allow the user to change and customize the widget settings.
     *
     * @since 1.0.0
     * @access protected
     */
    protected function register_controls()
    {


        $this->start_controls_section('section_progress', [
            'label' => __('Progress Bar', 'elementor'),
        ]);

        $this->add_control('progress_important_note', [
            'type' => Controls_Manager::RAW_HTML,
            'raw' => __(
                '<p>The following variables can be used in the title component to add dynamical content.</p><p><strong>{percent}</strong> The current percentage as shown in the progress bar.</p><p><strong>{number}</strong> The absolute number</p><p><strong>{network}</strong> The selected network.</p>',
                'photobooth-gallery'
            ),
        ]);

        $this->add_control('title', [
            'label' => __('Title', 'elementor'),
            'type' => Controls_Manager::TEXT,
            'dynamic' => [
                'active' => true,
            ],
            'placeholder' => __('Enter your title', 'elementor'),
            'default' => __('My Skill', 'elementor'),
            'label_block' => true,
        ]);

        $this->add_control(
            'title_tag',
            [
                'label' => esc_html__('Title HTML Tag', 'elementor'),
                'type' => Controls_Manager::SELECT,
                'options' => [
                    'h1' => 'H1',
                    'h2' => 'H2',
                    'h3' => 'H3',
                    'h4' => 'H4',
                    'h5' => 'H5',
                    'h6' => 'H6',
                    'div' => 'div',
                    'span' => 'span',
                    'p' => 'p',
                ],
                'default' => 'span',
                'condition' => [
                    'title!' => '',
                ],
            ]
        );

        $this->add_control(
            'progress_type',
            [
                'label' => esc_html__('Type', 'elementor'),
                'type' => Controls_Manager::SELECT,
                'default' => '',
                'options' => [
                    '' => esc_html__('Default', 'elementor'),
                    'info' => esc_html__('Info', 'elementor'),
                    'success' => esc_html__('Success', 'elementor'),
                    'warning' => esc_html__('Warning', 'elementor'),
                    'danger' => esc_html__('Danger', 'elementor'),
                ],
                'separator' => 'before',
            ]
        );

        $this->add_control('data_selector', [
            'label' => __('Data Type', 'photobooth-gallery'),
            'type' => Controls_Manager::SELECT,
            'default' => 'download',
            'options' => $this->data_types,
        ]);

        $this->add_control('view_selector', [
            'label' => __('View Type', 'photobooth-gallery'),
            'type' => Controls_Manager::SELECT,
            'default' => SocialMedia::SHARE,
            'options' => [
                SocialMedia::SHARE => __('Shared', 'photobooth-gallery'),
                SocialMedia::VIEW => __('Viewed', 'photobooth-gallery'),
                'unique' => __('Unique', 'photobooth-gallery'),
            ],
        ]);

        $this->add_control('auto_hide', [
            'label' => __('Auto Hide on empty values', 'photobooth-gallery'),
            'type' => Controls_Manager::SELECT,
            'default' => 'no',
            'options' => [
                'yes' => __('Yes', 'photobooth-gallery'),
                'no' => __('No', 'photobooth-gallery'),
            ],
        ]);

        $this->add_control('percent', [
            'label' => 'will be filled dynamically',
            'type' => Controls_Manager::HIDDEN,
            'default' => [
                'size' => 50,
                'unit' => '%',
            ],

        ]);

        $this->add_control(
            'display_percentage',
            [
                'label' => esc_html__('Display Percentage', 'elementor'),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__('Show', 'elementor'),
                'label_off' => esc_html__('Hide', 'elementor'),
                'return_value' => 'show',
                'default' => 'show',
            ]
        );

        $this->add_control(
            'inner_text',
            [
                'label' => esc_html__('Inner Text', 'elementor'),
                'type' => Controls_Manager::TEXT,
                'dynamic' => [
                    'active' => true,
                ],
                'placeholder' => esc_html__('e.g. Web Designer', 'elementor'),
                'default' => esc_html__('Web Designer', 'elementor'),
                'label_block' => true,
            ]
        );

        $this->add_control('view', [
            'label' => __('View', 'elementor'),
            'type' => Controls_Manager::HIDDEN,
            'default' => 'traditional',
        ]);

        $this->end_controls_section();

        $this->start_controls_section('section_progress_style', [
            'label' => __('Progress Bar', 'elementor'),
            'tab' => Controls_Manager::TAB_STYLE,
        ]);

        $this->add_control(
            'bar_color',
            [
                'label' => esc_html__('Color', 'elementor'),
                'type' => Controls_Manager::COLOR,
                'global' => [
                    'default' => Global_Colors::COLOR_PRIMARY,
                ],
                'selectors' => [
                    '{{WRAPPER}} .elementor-progress-wrapper .elementor-progress-bar' => 'background-color: {{VALUE}};',
                ],
            ]
        );

        $this->add_control(
            'bar_bg_color',
            [
                'label' => esc_html__('Background Color', 'elementor'),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .elementor-progress-wrapper' => 'background-color: {{VALUE}};',
                ],
            ]
        );

        $this->add_control(
            'bar_height',
            [
                'label' => esc_html__('Height', 'elementor'),
                'type' => Controls_Manager::SLIDER,
                'selectors' => [
                    '{{WRAPPER}} .elementor-progress-bar' => 'height: {{SIZE}}{{UNIT}}; line-height: {{SIZE}}{{UNIT}};',
                ],
            ]
        );
        $this->add_control(
            'bar_border_radius',
            [
                'label' => esc_html__('Border Radius', 'elementor'),
                'type' => Controls_Manager::SLIDER,
                'size_units' => ['px', '%', 'em', 'rem', 'custom'],
                'selectors' => [
                    '{{WRAPPER}} .elementor-progress-wrapper' => 'border-radius: {{SIZE}}{{UNIT}}; overflow: hidden;',
                ],
            ]
        );

        $this->add_control(
            'inner_text_heading',
            [
                'label' => esc_html__('Inner Text', 'elementor'),
                'type' => Controls_Manager::HEADING,
                'separator' => 'before',
            ]
        );

        $this->add_control('bar_inline_color', [
            'label' => __('Inner Text Color', 'elementor'),
            'type' => Controls_Manager::COLOR,
            'selectors' => [
                '{{WRAPPER}} .elementor-progress-bar' => 'color: {{VALUE}};',
            ],
        ]);

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'bar_inner_typography',
                'selector' => '{{WRAPPER}} .elementor-progress-bar',
                'exclude' => [
                    'line_height',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Text_Shadow::get_type(),
            [
                'name' => 'bar_inner_shadow',
                'selector' => '{{WRAPPER}} .elementor-progress-bar',
            ]
        );

        $this->end_controls_section();

        $this->start_controls_section(
            'section_title',
            [
                'label' => esc_html__('Title Style', 'elementor'),
                'tab' => Controls_Manager::TAB_STYLE,
            ]
        );

        $this->add_control(
            'title_color',
            [
                'label' => esc_html__('Text Color', 'elementor'),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .elementor-title' => 'color: {{VALUE}};',
                ],
                'global' => [
                    'default' => Global_Colors::COLOR_PRIMARY,
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'typography',
                'selector' => '{{WRAPPER}} .elementor-title',
                'global' => [
                    'default' => Global_Typography::TYPOGRAPHY_TEXT,
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Text_Shadow::get_type(),
            [
                'name' => 'title_shadow',
                'selector' => '{{WRAPPER}} .elementor-title',
            ]
        );

        $this->end_controls_section();
    }

    private function get_value_from_service($settings)
    {

        $service = new SocialMediaStatistics();
        $engine = new Engine();

        $service->setCode(get_query_var('pg-code'));

        $count_by = '_all';

        // Xml Data has other maximum counts
        if (strpos($settings['data_selector'], 'xml-') !== false) {
            $count_by = 'xml-fotos';
        }

        // Frontend Views have other maximum counts
        if (strpos($settings['data_selector'], '-clicked') !== false) {
            $count_by = 'all-viewed';
        }

        $all = $service->count_by_network($count_by, $settings['view_selector']);
        $specific = 0;

        if ($all <= 0) {
            $all = 0.001;
        } else {
            $specific = $service->count_by_network($settings['data_selector'], $settings['view_selector']);
        }

        $title = $engine->render($settings['title'], [
            'percent' => sprintf('%.2f%%', ($specific / $all) * 100),
            'number' => number_format($specific, 0, ',', '.'),
            'network' => $this->data_types[$settings['data_selector']] ?? __('N/A', 'photobooth-gallery'),
        ]);

        $response = compact('all', 'specific', 'title');

        return $response;
    }


    /**
     * Render progress widget output on the frontend.
     *
     * Written in PHP and used to generate the final HTML.
     *
     * @since 1.0.0
     * @access protected
     */
    protected function render()
    {
        $settings = $this->get_settings_for_display();

        $stats = $this->get_value_from_service($settings);

        $settings['percent']['size'] = round(($stats['specific'] / ($stats['all'] + 0.001)) * 100);
        $settings['title'] = $stats['title'];

        if ($stats['specific'] <= 0 && $settings['auto_hide'] === 'yes') {
            return;
        }

        $this->add_inline_editing_attributes('title');

        $this->add_render_attribute('wrapper', [
            'class' => 'elementor-progress-wrapper',
            'role' => 'progressbar',
            'aria-valuemin' => '0',
            'aria-valuemax' => '100',
            'aria-valuenow' => $settings['percent']['size'],
            'aria-valuetext' => $settings['inner_text'],
        ]);

        if (!empty($settings['progress_type'])) {
            $this->add_render_attribute('wrapper', 'class', 'progress-'.$settings['progress_type']);
        }

        $this->add_render_attribute('_wrapper', [
            'data-autohide' => true,
            'data-value' => $settings['percent']['size'],
        ]);

        $this->add_render_attribute('progress-bar', [
            'class' => 'elementor-progress-bar',
            'data-max' => $settings['percent']['size'],
        ]);

        $this->add_render_attribute('inner_text', [
            'class' => 'elementor-progress-text',
        ]);

        $this->add_inline_editing_attributes('inner_text');

        if (!empty($settings['title'])) { ?>
            <span class="elementor-title"><?php echo $settings['title']; ?></span>
        <?php } ?>

        <div <?php echo $this->get_render_attribute_string('wrapper'); ?>>
            <div <?php echo $this->get_render_attribute_string('progress-bar'); ?>>
                <span <?php $this->print_render_attribute_string('inner_text'); ?>><?php $this->print_unescaped_setting(
                        'inner_text'
                    ); ?></span>
                <?php if ('hide' !== $settings['display_percentage']) { ?>
                    <span class="elementor-progress-percentage"><?php echo $settings['percent']['size']; ?>%</span>
                <?php } ?>
            </div>
        </div>
        <?php
    }

}
