<?php

function wp_date_localised( $format, $timestamp = null ) {
    // This function behaves a bit like PHP's Date() function, but taking into account the Wordpress site's timezone
    // CAUTION: It will throw an exception when it receives invalid input - please catch it accordingly
    // From https://mediarealm.com.au/

    $tz_string = get_option( 'timezone_string' );
    $tz_offset = get_option( 'gmt_offset', 0 );

    if ( ! empty( $tz_string ) ) {
        // If site timezone option string exists, use it
        $timezone = $tz_string;

    } elseif ( $tz_offset == 0 ) {
        // get UTC offset, if it isn’t set then return UTC
        $timezone = 'UTC';

    } else {
        $timezone = $tz_offset;

        if ( substr( $tz_offset, 0, 1 ) != "-" && substr( $tz_offset, 0, 1 ) != "+" && substr( $tz_offset, 0, 1 ) != "U" ) {
            $timezone = "+" . $tz_offset;
        }
    }

    if ( $timestamp === null ) {
        $timestamp = time();
    }

    $datetime = new DateTime();
    $datetime->setTimestamp( $timestamp );
    $datetime->setTimezone( new DateTimeZone( $timezone ) );

    return $datetime->format( $format );
}

add_action( 'init',
    function() {

        $params = [];

        foreach ( $_GET as $key => $value ) {
            $params[ sanitize_title( $key ) ] = sanitize_text_field( $value );
        }

        if ( ! $params ) {
            return;
        }

        foreach ( $params as $shortcode => $value ) {

            add_shortcode( $shortcode,
                function( $atts ) use ( $shortcode, $value ) {

                    $atts = shortcode_atts( array(
                        'default' => '',
                    ),
                        $atts,
                        $shortcode );

                    if ( ! $value ) {
                        return $atts[ 'default' ];
                    }

                    return $value;
                } );
        }

    } );


add_action( 'init',
    function() {

        add_shortcode( 'cv-edit',
            function( $atts ) {

                $atts = shortcode_atts( [ 'form_id' => 0 ], $atts, 'cv-edit' );

                $form_id = $atts[ 'form_id' ];

                if ( ! $form_id ) {
                    return;
                }

                $user_id = get_current_user_id();

                if ( $user_id === 0 ) {
                    wp_redirect( '/auftragsabwicklung/' );

                    return;
                }

                $entry_log = get_post_meta( $user_id, 'cv-entry-log', true );

                if ( ! $entry_log ) {
                    wp_redirect( '/auftragsabwicklung/' );

                    return;
                }

                $entry_log = end( $entry_log );

                $entry = GFAPI::get_entry( $entry_log[ 'last-id' ] );

                $converted = [];
                foreach ( $entry as $key => $val ) {
                    $converted[ 'input_' . $key ] = $val;
                }

                $converted[ 'input_lfn' ] = sprintf( '%08d', sizeof( $entry_log ) + 1 );


                gravity_form( $form_id, false, false, false, $converted, true );

            } );
    } );
add_action( 'gform_pre_render_1', function( $form ) {
    $user_id = get_current_user_id();

    if ( $user_id === 0 ) {
        return $form;
    }

    $entry_log = get_post_meta( $user_id, 'cv-entry-log', true );

    if ( ! $entry_log ) {
        return $form;
    }

    $entry_log = end( $entry_log );

    foreach ( $form[ "fields" ] as &$field ) {

        if ( $field[ "id" ] !== 1 ) {
            continue;
        }

        $field[ "content" ] = 'Letzte Änderung: ' . wp_date_localised($entry_log['last-change']);
    }

    return $form;

} );
add_action( 'gform_after_submission_1',
    function( $entry, $form ) {

        $entry[ 'post_id' ];
        $user_id = get_current_user_id();

        $entry_log = get_post_meta( $user_id, 'cv-entry-log', true );

        if ( ! $entry_log ) {
            $entry_log = [];
        }

        $entry_log[] = [
            'last-user'   => get_current_user_id(),
            'last-change' => time(),
            'last-id'     => $entry[ 'post_id' ],
        ];

        update_post_meta( $user_id, 'cv-entry-log', $entry_log );
    } );
