<?php
/**
 * Created by PhpStorm.
 * User: b2wsra
 * Date: 09.10.2018
 * Time: 17:03
 *
 * @license proprietary?
 * Modified by sebastian rassmann on 06-February-2024 using Strauss.
 * @see https://github.com/BrianHenryIE/strauss
 */

namespace Codevision\Language;

use Photobooth\StringTemplate\Engine;

class ACFPolyLang
{

    private string $text_domain;

    private array $slugs;

    private array $data;

    private Engine $engine;

    public function __construct(string $text_domain, array $slugs, array $data)
    {
        $this->text_domain = $text_domain;
        $this->slugs = $slugs;
        $this->data = $data;

        $this->engine = new Engine();

    }


    public function init()
    {

        add_action('current_screen', [$this, 'init_screen']);

    }

    function init_screen($screen)
    {

        if (!is_admin()) {
            return;
        }

        $enable_filter = array_filter($this->slugs, function ($slug) use ($screen) {
            return strpos($screen->id, $slug) !== false;
        });

        if (!$enable_filter) {
            return;
        }

        add_filter('acf/load_field', [$this, 'load_field']);
        add_filter('acf/prepare_field', [$this, 'load_field']);
    }

    function load_field($field)
    {

        $text_fields = [
            'label',
            'instructions',
            'message',
        ];

        foreach ($text_fields as $label) {

            if (!array_key_exists($label, $field)) {
                continue;
            }

            $field[$label] = $this->trans($field[$label]);
        }


        if (array_key_exists('ui_on_text', $field)) {
            $field['ui_on_text'] = __('Yes', $this->text_domain);
        }

        if (array_key_exists('ui_off_text', $field)) {
            $field['ui_off_text'] = __('No', $this->text_domain);
        }

        if (array_key_exists('choices', $field)) {

            $choices = $field['choices'];

            foreach ($choices as $key => & $desc) {
                $desc = $this->trans($desc);
            }

            $field['choices'] = $choices;
        }


        return $field;
    }

    private function trans($label)
    {
        return $this->engine->render(__($label, $this->text_domain), $this->data);
    }
}
