<?php

/*
 * This file is part of PHP-FFmpeg.
 *
 * (c) Alchemy <dev.team@alchemy.fr>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 *
 * Modified by sebastian rassmann on 06-February-2024 using Strauss.
 * @see https://github.com/BrianHenryIE/strauss
 */

namespace Photobooth\FFMpeg\Filters\Audio;

use Photobooth\FFMpeg\Format\AudioInterface;
use Photobooth\FFMpeg\Media\Audio;

class CustomFilter implements AudioFilterInterface
{
    /** @var string */
    private $filter;
    /** @var int */
    private $priority;

    /**
     * A custom filter, useful if you want to build complex filters.
     *
     * @param string $filter
     * @param int    $priority
     */
    public function __construct($filter, $priority = 0)
    {
        $this->filter = $filter;
        $this->priority = $priority;
    }

    /**
     * {@inheritdoc}
     */
    public function getPriority()
    {
        return $this->priority;
    }

    /**
     * {@inheritdoc}
     */
    public function apply(Audio $audio, AudioInterface $format)
    {
        $commands = ['-af', $this->filter];

        return $commands;
    }
}
